@@ -10,13 +10,21 @@
 #
 # All configuration values have a default value; values that are commented out
 # serve to show the default value.
-#
-# $Id: conf.py,v 1.21 2015/06/05 21:05:37 stroeder Exp $
 
 import sys
+import os
 
 # If your extensions are in another directory, add it here.
-#sys.path.append('some/directory')
+_doc_dir = os.path.dirname(__file__)
+sys.path.append(_doc_dir)
+sys.path.insert(0, os.path.join(_doc_dir, '../Lib/'))
+sys.path.insert(0, os.path.join(_doc_dir, '../Lib/ldap'))
+
+# Import fake `_ldap` module
+import fake_ldap_module_for_documentation
+
+# Now ldap can be used normally
+from ldap import __version__
 
 # General configuration
 # ---------------------
@@ -25,6 +33,13 @@ import sys
 # coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
 extensions = ['sphinx.ext.autodoc']
 
+try:
+    import sphinxcontrib.spelling
+except ImportError:
+    pass
+else:
+    extensions.append('sphinxcontrib.spelling')
+
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ['.templates']
 
@@ -36,15 +51,15 @@ master_doc = 'index'
 
 # General substitutions.
 project = 'python-ldap'
-copyright = '2008-2014, python-ldap project team'
+copyright = '2008-2017, python-ldap project team'
 
 # The default replacements for |version| and |release|, also used in various
 # other places throughout the built documents.
 #
 # The short X.Y version.
-version = '2.4'
+version = '.'.join(__version__.split('.')[:2])
 # The full version, including alpha/beta/rc tags.
-release = '2.4.20.0'
+release = __version__
 
 # There are two options for replacing |today|: either, you set today to some
 # non-false value, then it is used:
@@ -76,12 +91,12 @@ pygments_style = 'sphinx'
 # The style sheet to use for HTML and HTML Help pages. A file of that name
 # must exist either in Sphinx' static/ path, or in one of the custom paths
 # given in html_static_path.
-html_style = 'default.css'
+#html_style = 'pyramid.css'
 
 # Add any paths that contain custom static files (such as style sheets) here,
 # relative to this directory. They are copied after the builtin static files,
 # so a file named "default.css" will overwrite the builtin "default.css".
-html_static_path = ['.static']
+#html_static_path = ['/usr/lib/python2.7/site-packages/sphinx/themes/pyramid/static']
 
 # If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
 # using the given strftime format.
