@@ -2,31 +2,19 @@
 ldapobject.py - wraps class _ldap.LDAPObject
 
 See https://www.python-ldap.org/ for details.
-
-\$Id: ldapobject.py,v 1.165 2017/09/04 14:59:12 stroeder Exp $
-
-Compability:
-- Tested with Python 2.0+ but should work with Python 1.5.x
-- LDAPObject class should be exactly the same like _ldap.LDAPObject
-
-Usage:
-Directly imported by ldap/__init__.py. The symbols of _ldap are
-overridden.
-
-Thread-lock:
-Basically calls into the LDAP lib are serialized by the module-wide
-lock self._ldap_object_lock.
 """
 
+from __future__ import unicode_literals
+
 from os import strerror
 
-from ldap import __version__
+from ldap.pkginfo import __version__, __author__, __license__
 
 __all__ = [
   'LDAPObject',
   'SimpleLDAPObject',
-  'NonblockingLDAPObject',
   'ReconnectLDAPObject',
+  'LDAPBytesWarning'
 ]
 
 
@@ -35,13 +23,49 @@ if __debug__:
   import traceback
 
 import sys,time,pprint,_ldap,ldap,ldap.sasl,ldap.functions
+import warnings
 
 from ldap.schema import SCHEMA_ATTRS
 from ldap.controls import LDAPControl,DecodeControlTuples,RequestControlTuples
 from ldap.extop import ExtendedRequest,ExtendedResponse
+from ldap.compat import reraise
 
 from ldap import LDAPError
 
+PY2 = sys.version_info[0] <= 2
+if PY2:
+  text_type = unicode
+else:
+  text_type = str
+
+
+# See SimpleLDAPObject._bytesify_input
+_LDAP_WARN_SKIP_FRAME = True
+
+class LDAPBytesWarning(BytesWarning):
+  """python-ldap bytes mode warning
+  """
+
+def _raise_byteswarning(message):
+    """Raise LDAPBytesWarning
+    """
+
+    # Call stacks that raise the warning tend to be complicated, so
+    # getting a useful stacklevel is tricky.
+    # We walk stack frames, ignoring functions in uninteresting files,
+    # based on the _LDAP_WARN_SKIP_FRAME marker in globals().
+    stacklevel = 2
+    try:
+        getframe = sys._getframe
+    except AttributeError:
+        pass
+    else:
+        frame = sys._getframe(stacklevel)
+        while frame and frame.f_globals.get('_LDAP_WARN_SKIP_FRAME'):
+            stacklevel += 1
+            frame = frame.f_back
+    warnings.warn(message, LDAPBytesWarning, stacklevel=stacklevel+1)
+
 
 class NO_UNIQUE_ENTRY(ldap.NO_SUCH_OBJECT):
   """
@@ -69,7 +93,8 @@ class SimpleLDAPObject:
 
   def __init__(
     self,uri,
-    trace_level=0,trace_file=None,trace_stack_limit=5
+    trace_level=0,trace_file=None,trace_stack_limit=5,bytes_mode=None,
+    bytes_strictness=None,
   ):
     self._trace_level = trace_level
     self._trace_file = trace_file or sys.stdout
@@ -80,6 +105,186 @@ class SimpleLDAPObject:
     self.timeout = -1
     self.protocol_version = ldap.VERSION3
 
+    # Bytes mode
+    # ----------
+
+    if PY2:
+        if bytes_mode is None:
+            bytes_mode = True
+            if bytes_strictness is None:
+                _raise_byteswarning(
+                  "Under Python 2, python-ldap uses bytes by default. "
+                  "This will be removed in Python 3 (no bytes for "
+                  "DN/RDN/field names). "
+                  "Please call initialize(..., bytes_mode=False) explicitly.")
+                bytes_strictness = 'warn'
+        else:
+            if bytes_strictness is None:
+                bytes_strictness = 'error'
+    else:
+        if bytes_mode:
+            raise ValueError("bytes_mode is *not* supported under Python 3.")
+        bytes_mode = False
+        bytes_strictness = 'error'
+    self.bytes_mode = bytes_mode
+    self.bytes_strictness = bytes_strictness
+
+  def _bytesify_input(self, arg_name, value):
+    """Adapt a value following bytes_mode in Python 2.
+
+    In Python 3, returns the original value unmodified.
+
+    With bytes_mode ON, takes bytes or None and returns bytes or None.
+    With bytes_mode OFF, takes unicode or None and returns bytes or None.
+
+    For the wrong argument type (unicode or bytes, respectively),
+    behavior depends on the bytes_strictness setting.
+    In all cases, bytes or None are returned (or an exception is raised).
+    """
+    if not PY2:
+      return value
+    if value is None:
+      return value
+
+    elif self.bytes_mode:
+      if isinstance(value, bytes):
+        return value
+      elif self.bytes_strictness == 'silent':
+        pass
+      elif self.bytes_strictness == 'warn':
+        _raise_byteswarning(
+            "Received non-bytes value for '{}' in bytes mode; "
+            "please choose an explicit "
+            "option for bytes_mode on your LDAP connection".format(arg_name))
+      else:
+          raise TypeError(
+            "All provided fields *must* be bytes when bytes mode is on; "
+            "got type '{}' for '{}'.".format(type(value).__name__, arg_name)
+          )
+      return value.encode('utf-8')
+    else:
+      if isinstance(value, unicode):
+        return value.encode('utf-8')
+      elif self.bytes_strictness == 'silent':
+        pass
+      elif self.bytes_strictness == 'warn':
+        _raise_byteswarning(
+            "Received non-text value for '{}' with bytes_mode off and "
+            "bytes_strictness='warn'".format(arg_name))
+      else:
+        raise TypeError(
+          "All provided fields *must* be text when bytes mode is off; "
+          "got type '{}' for '{}'.".format(type(value).__name__, arg_name)
+        )
+      return value
+
+  def _bytesify_modlist(self, arg_name, modlist, with_opcode):
+    """Adapt a modlist according to bytes_mode.
+
+    A modlist is a tuple of (op, attr, value), where:
+    - With bytes_mode ON, attr is checked to be bytes
+    - With bytes_mode OFF, attr is converted from unicode to bytes
+    - value is *always* bytes
+    """
+    if not PY2:
+      return modlist
+    if with_opcode:
+      return tuple(
+        (op, self._bytesify_input(arg_name, attr), val)
+        for op, attr, val in modlist
+      )
+    else:
+      return tuple(
+        (self._bytesify_input(arg_name, attr), val)
+        for attr, val in modlist
+      )
+
+  def _unbytesify_text_value(self, value):
+    """Adapt a 'known text, UTF-8 encoded' returned value following bytes_mode.
+
+    With bytes_mode ON, takes bytes or None and returns bytes or None.
+    With bytes_mode OFF, takes bytes or None and returns unicode or None.
+
+    This function should only be applied on field *values*; distinguished names
+    or field *names* are already natively handled in result4.
+    """
+    if value is None:
+      return value
+
+    # Preserve logic of assertions only under Python 2
+    if PY2:
+      assert isinstance(value, bytes), "Expected bytes value, got text instead (%r)" % (value,)
+
+    if self.bytes_mode:
+      return value
+    else:
+      return value.decode('utf-8')
+
+  def _maybe_rebytesify_text(self, value):
+    """Re-encodes text to bytes if needed by bytes_mode.
+
+    Takes unicode (and checks for it), and returns:
+    - bytes under bytes_mode
+    - unicode otherwise.
+    """
+    if not PY2:
+      return value
+
+    if value is None:
+      return value
+
+    assert isinstance(value, text_type), "Should return text, got bytes instead (%r)" % (value,)
+    if not self.bytes_mode:
+      return value
+    else:
+      return value.encode('utf-8')
+
+  def _bytesify_result_value(self, result_value):
+    """Applies bytes_mode to a result value.
+
+    Such a value can either be:
+    - a dict mapping an attribute name to its list of values
+      (where attribute names are unicode and values bytes)
+    - a list of referals (which are unicode)
+    """
+    if not PY2:
+      return result_value
+    if hasattr(result_value, 'items'):
+      # It's a attribute_name: [values] dict
+      return {
+        self._maybe_rebytesify_text(key): value
+        for (key, value) in result_value.items()
+      }
+    elif isinstance(result_value, bytes):
+      return result_value
+    else:
+      # It's a list of referals
+      # Example value:
+      # [u'ldap://DomainDnsZones.xxxx.root.local/DC=DomainDnsZones,DC=xxxx,DC=root,DC=local']
+      return [self._maybe_rebytesify_text(referal) for referal in result_value]
+
+  def _bytesify_results(self, results, with_ctrls=False):
+    """Converts a "results" object according to bytes_mode.
+
+    Takes:
+    - a list of (dn, {field: [values]}) if with_ctrls is False
+    - a list of (dn, {field: [values]}, ctrls) if with_ctrls is True
+
+    And, if bytes_mode is on, converts dn and fields to bytes.
+    """
+    if not PY2:
+      return results
+    if with_ctrls:
+      return [
+        (self._maybe_rebytesify_text(dn), self._bytesify_result_value(fields), ctrls)
+        for (dn, fields, ctrls) in results
+      ]
+    else:
+      return [
+        (self._maybe_rebytesify_text(dn), self._bytesify_result_value(fields))
+        for (dn, fields) in results
+      ]
+
   def _ldap_lock(self,desc=''):
     if ldap.LIBLDAP_R:
       return ldap.LDAPLock(desc='%s within %s' %(desc,repr(self)))
@@ -111,15 +316,19 @@ class SimpleLDAPObject:
             diagnostic_message_success = self._l.get_option(ldap.OPT_DIAGNOSTIC_MESSAGE)
       finally:
         self._ldap_object_lock.release()
-    except LDAPError, e:
+    except LDAPError as e:
+      exc_type,exc_value,exc_traceback = sys.exc_info()
       try:
-        if not e.args[0].has_key('info') and e.args[0].has_key('errno'):
+        if 'info' not in e.args[0] and 'errno' in e.args[0]:
           e.args[0]['info'] = strerror(e.args[0]['errno'])
       except IndexError:
         pass
       if __debug__ and self._trace_level>=2:
         self._trace_file.write('=> LDAPError - %s: %s\n' % (e.__class__.__name__,str(e)))
-      raise
+      try:
+        reraise(exc_type, exc_value, exc_traceback)
+      finally:
+        exc_type = exc_value = exc_traceback = None
     else:
       if __debug__ and self._trace_level>=2:
         if not diagnostic_message_success is None:
@@ -128,20 +337,20 @@ class SimpleLDAPObject:
     return result
 
   def __setattr__(self,name,value):
-    if self.CLASSATTR_OPTION_MAPPING.has_key(name):
+    if name in self.CLASSATTR_OPTION_MAPPING:
       self.set_option(self.CLASSATTR_OPTION_MAPPING[name],value)
     else:
       self.__dict__[name] = value
 
   def __getattr__(self,name):
-    if self.CLASSATTR_OPTION_MAPPING.has_key(name):
+    if name in self.CLASSATTR_OPTION_MAPPING:
       return self.get_option(self.CLASSATTR_OPTION_MAPPING[name])
-    elif self.__dict__.has_key(name):
+    elif name in self.__dict__:
       return self.__dict__[name]
     else:
-      raise AttributeError,'%s has no attribute %s' % (
+      raise AttributeError('%s has no attribute %s' % (
         self.__class__.__name__,repr(name)
-      )
+      ))
 
   def fileno(self):
     """
@@ -195,6 +404,9 @@ class SimpleLDAPObject:
         The parameter modlist is similar to the one passed to modify(),
         except that no operation integer need be included in the tuples.
     """
+    if PY2:
+        dn = self._bytesify_input('dn', dn)
+        modlist = self._bytesify_modlist('modlist', modlist, with_opcode=False)
     return self._ldap_call(self._l.add_ext,dn,modlist,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def add_ext_s(self,dn,modlist,serverctrls=None,clientctrls=None):
@@ -215,13 +427,16 @@ class SimpleLDAPObject:
   def add_s(self,dn,modlist):
     return self.add_ext_s(dn,modlist,None,None)
 
-  def simple_bind(self,who='',cred='',serverctrls=None,clientctrls=None):
+  def simple_bind(self,who=None,cred=None,serverctrls=None,clientctrls=None):
     """
     simple_bind([who='' [,cred='']]) -> int
     """
+    if PY2:
+        who = self._bytesify_input('who', who)
+        cred = self._bytesify_input('cred', cred)
     return self._ldap_call(self._l.simple_bind,who,cred,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
-  def simple_bind_s(self,who='',cred='',serverctrls=None,clientctrls=None):
+  def simple_bind_s(self,who=None,cred=None,serverctrls=None,clientctrls=None):
     """
     simple_bind_s([who='' [,cred='']]) -> 4-tuple
     """
@@ -295,17 +510,22 @@ class SimpleLDAPObject:
         A design bug in the library prevents value from containing
         nul characters.
     """
+    if PY2:
+        dn = self._bytesify_input('dn', dn)
+        attr = self._bytesify_input('attr', attr)
     return self._ldap_call(self._l.compare_ext,dn,attr,value,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def compare_ext_s(self,dn,attr,value,serverctrls=None,clientctrls=None):
     msgid = self.compare_ext(dn,attr,value,serverctrls,clientctrls)
     try:
-      resp_type, resp_data, resp_msgid, resp_ctrls = self.result3(msgid,all=1,timeout=self.timeout)
+        ldap_res = self.result3(msgid,all=1,timeout=self.timeout)
     except ldap.COMPARE_TRUE:
       return 1
     except ldap.COMPARE_FALSE:
       return 0
-    return None
+    raise ldap.PROTOCOL_ERROR(
+        'Compare operation returned wrong result: %r' % (ldap_res)
+    )
 
   def compare(self,dn,attr,value):
     return self.compare_ext(dn,attr,value,None,None)
@@ -318,11 +538,12 @@ class SimpleLDAPObject:
     delete(dn) -> int
     delete_s(dn) -> None
     delete_ext(dn[,serverctrls=None[,clientctrls=None]]) -> int
-    delete_ext_s(dn[,serverctrls=None[,clientctrls=None]]) -> None
+    delete_ext_s(dn[,serverctrls=None[,clientctrls=None]]) -> tuple
         Performs an LDAP delete operation on dn. The asynchronous
         form returns the message id of the initiated request, and the
         result can be obtained from a subsequent call to result().
     """
+    dn = self._bytesify_input('dn', dn)
     return self._ldap_call(self._l.delete_ext,dn,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def delete_ext_s(self,dn,serverctrls=None,clientctrls=None):
@@ -371,6 +592,9 @@ class SimpleLDAPObject:
     """
     modify_ext(dn, modlist[,serverctrls=None[,clientctrls=None]]) -> int
     """
+    if PY2:
+        dn = self._bytesify_input('dn', dn)
+        modlist = self._bytesify_modlist('modlist', modlist, with_opcode=True)
     return self._ldap_call(self._l.modify_ext,dn,modlist,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def modify_ext_s(self,dn,modlist,serverctrls=None,clientctrls=None):
@@ -383,7 +607,7 @@ class SimpleLDAPObject:
     modify(dn, modlist) -> int
     modify_s(dn, modlist) -> None
     modify_ext(dn, modlist[,serverctrls=None[,clientctrls=None]]) -> int
-    modify_ext_s(dn, modlist[,serverctrls=None[,clientctrls=None]]) -> None
+    modify_ext_s(dn, modlist[,serverctrls=None[,clientctrls=None]]) -> tuple
         Performs an LDAP modify operation on an entry's attributes.
         dn is the DN of the entry to modify, and modlist is the list
         of modifications to make to the entry.
@@ -424,6 +648,10 @@ class SimpleLDAPObject:
     return self.rename_s(dn,newrdn,None,delold)
 
   def passwd(self,user,oldpw,newpw,serverctrls=None,clientctrls=None):
+    if PY2:
+        user = self._bytesify_input('user', user)
+        oldpw = self._bytesify_input('oldpw', oldpw)
+        newpw = self._bytesify_input('newpw', newpw)
     return self._ldap_call(self._l.passwd,user,oldpw,newpw,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def passwd_s(self,user,oldpw,newpw,serverctrls=None,clientctrls=None):
@@ -445,6 +673,10 @@ class SimpleLDAPObject:
         This actually corresponds to the rename* routines in the
         LDAP-EXT C API library.
     """
+    if PY2:
+        dn = self._bytesify_input('dn', dn)
+        newrdn = self._bytesify_input('newrdn', newrdn)
+        newsuperior = self._bytesify_input('newsuperior', newsuperior)
     return self._ldap_call(self._l.rename,dn,newrdn,newsuperior,delold,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
 
   def rename_s(self,dn,newrdn,newsuperior=None,delold=1,serverctrls=None,clientctrls=None):
@@ -458,7 +690,7 @@ class SimpleLDAPObject:
 
         This method is used to wait for and return the result of an
         operation previously initiated by one of the LDAP asynchronous
-        operation routines (eg search(), modify(), etc.) They all
+        operation routines (e.g. search(), modify(), etc.) They all
         returned an invocation identifier (a message id) upon successful
         initiation of their operation. This id is guaranteed to be
         unique across an LDAP session, and can be used to request the
@@ -533,9 +765,11 @@ class SimpleLDAPObject:
       if add_ctrls:
         resp_data = [ (t,r,DecodeControlTuples(c,resp_ctrl_classes)) for t,r,c in resp_data ]
     decoded_resp_ctrls = DecodeControlTuples(resp_ctrls,resp_ctrl_classes)
+    if resp_data is not None:
+        resp_data = self._bytesify_results(resp_data, with_ctrls=add_ctrls)
     return resp_type, resp_data, resp_msgid, decoded_resp_ctrls, resp_name, resp_value
 
-  def search_ext(self,base,scope,filterstr='(objectClass=*)',attrlist=None,attrsonly=0,serverctrls=None,clientctrls=None,timeout=-1,sizelimit=0):
+  def search_ext(self,base,scope,filterstr=None,attrlist=None,attrsonly=0,serverctrls=None,clientctrls=None,timeout=-1,sizelimit=0):
     """
     search(base, scope [,filterstr='(objectClass=*)' [,attrlist=None [,attrsonly=0]]]) -> int
     search_s(base, scope [,filterstr='(objectClass=*)' [,attrlist=None [,attrsonly=0]]])
@@ -580,6 +814,24 @@ class SimpleLDAPObject:
         The amount of search results retrieved can be limited with the
         sizelimit parameter if non-zero.
     """
+
+    if PY2:
+        base = self._bytesify_input('base', base)
+        if filterstr is None:
+          # workaround for default argument,
+          # see https://github.com/python-ldap/python-ldap/issues/147
+          if self.bytes_mode:
+            filterstr = b'(objectClass=*)'
+          else:
+            filterstr = u'(objectClass=*)'
+        else:
+          filterstr = self._bytesify_input('filterstr', filterstr)
+        if attrlist is not None:
+            attrlist = tuple(self._bytesify_input('attrlist', a)
+                             for a in attrlist)
+    else:
+      if filterstr is None:
+        filterstr = '(objectClass=*)'
     return self._ldap_call(
       self._l.search_ext,
       base,scope,filterstr,
@@ -589,17 +841,17 @@ class SimpleLDAPObject:
       timeout,sizelimit,
     )
 
-  def search_ext_s(self,base,scope,filterstr='(objectClass=*)',attrlist=None,attrsonly=0,serverctrls=None,clientctrls=None,timeout=-1,sizelimit=0):
+  def search_ext_s(self,base,scope,filterstr=None,attrlist=None,attrsonly=0,serverctrls=None,clientctrls=None,timeout=-1,sizelimit=0):
     msgid = self.search_ext(base,scope,filterstr,attrlist,attrsonly,serverctrls,clientctrls,timeout,sizelimit)
     return self.result(msgid,all=1,timeout=timeout)[1]
 
-  def search(self,base,scope,filterstr='(objectClass=*)',attrlist=None,attrsonly=0):
+  def search(self,base,scope,filterstr=None,attrlist=None,attrsonly=0):
     return self.search_ext(base,scope,filterstr,attrlist,attrsonly,None,None)
 
-  def search_s(self,base,scope,filterstr='(objectClass=*)',attrlist=None,attrsonly=0):
+  def search_s(self,base,scope,filterstr=None,attrlist=None,attrsonly=0):
     return self.search_ext_s(base,scope,filterstr,attrlist,attrsonly,None,None,timeout=self.timeout)
 
-  def search_st(self,base,scope,filterstr='(objectClass=*)',attrlist=None,attrsonly=0,timeout=-1):
+  def search_st(self,base,scope,filterstr=None,attrlist=None,attrsonly=0,timeout=-1):
     return self.search_ext_s(base,scope,filterstr,attrlist,attrsonly,None,None,timeout)
 
   def start_tls_s(self):
@@ -666,17 +918,27 @@ class SimpleLDAPObject:
       invalue = RequestControlTuples(invalue)
     return self._ldap_call(self._l.set_option,option,invalue)
 
-  def search_subschemasubentry_s(self,dn=''):
+  def search_subschemasubentry_s(self,dn=None):
     """
     Returns the distinguished name of the sub schema sub entry
     for a part of a DIT specified by dn.
 
     None as result indicates that the DN of the sub schema sub entry could
     not be determined.
+
+    Returns: None or text/bytes depending on bytes_mode.
     """
+    if self.bytes_mode:
+      empty_dn = b''
+      attrname = b'subschemaSubentry'
+    else:
+      empty_dn = u''
+      attrname = u'subschemaSubentry'
+    if dn is None:
+      dn = empty_dn
     try:
       r = self.search_s(
-        dn,ldap.SCOPE_BASE,'(objectClass=*)',['subschemaSubentry']
+        dn,ldap.SCOPE_BASE,None,[attrname]
       )
     except (ldap.NO_SUCH_OBJECT,ldap.NO_SUCH_ATTRIBUTE,ldap.INSUFFICIENT_ACCESS):
       r = []
@@ -685,16 +947,18 @@ class SimpleLDAPObject:
     try:
       if r:
         e = ldap.cidict.cidict(r[0][1])
-        search_subschemasubentry_dn = e.get('subschemaSubentry',[None])[0]
+        search_subschemasubentry_dn = e.get(attrname,[None])[0]
         if search_subschemasubentry_dn is None:
           if dn:
             # Try to find sub schema sub entry in root DSE
-            return self.search_subschemasubentry_s(dn='')
+            return self.search_subschemasubentry_s(dn=empty_dn)
           else:
             # If dn was already root DSE we can return here
             return None
         else:
-          return search_subschemasubentry_dn
+          # With legacy bytes mode, return bytes; otherwise, since this is a DN,
+          # RFCs impose that the field value *can* be decoded to UTF-8.
+          return self._unbytesify_text_value(search_subschemasubentry_dn)
     except IndexError:
       return None
 
@@ -707,7 +971,7 @@ class SimpleLDAPObject:
     r = self.search_ext_s(
       dn,
       ldap.SCOPE_BASE,
-      filterstr or '(objectClass=*)',
+      filterstr,
       attrlist=attrlist,
       serverctrls=serverctrls,
       clientctrls=clientctrls,
@@ -722,18 +986,26 @@ class SimpleLDAPObject:
     """
     Returns the sub schema sub entry's data
     """
+    if self.bytes_mode:
+      filterstr = b'(objectClass=subschema)'
+      if attrs is None:
+        attrs = [attr.encode('utf-8') for attr in SCHEMA_ATTRS]
+    else:
+      filterstr = u'(objectClass=subschema)'
+      if attrs is None:
+        attrs = SCHEMA_ATTRS
     try:
       subschemasubentry = self.read_s(
         subschemasubentry_dn,
-        filterstr='(objectClass=subschema)',
-        attrlist=attrs or SCHEMA_ATTRS
+        filterstr=filterstr,
+        attrlist=attrs
       )
     except ldap.NO_SUCH_OBJECT:
       return None
     else:
       return subschemasubentry
 
-  def find_unique_entry(self,base,scope=ldap.SCOPE_SUBTREE,filterstr='(objectClass=*)',attrlist=None,attrsonly=0,serverctrls=None,clientctrls=None,timeout=-1):
+  def find_unique_entry(self,base,scope=ldap.SCOPE_SUBTREE,filterstr=None,attrlist=None,attrsonly=0,serverctrls=None,clientctrls=None,timeout=-1):
     """
     Returns a unique entry, raises exception if not unique
     """
@@ -741,7 +1013,7 @@ class SimpleLDAPObject:
       base,
       scope,
       filterstr,
-      attrlist=attrlist or ['*'],
+      attrlist=attrlist,
       attrsonly=attrsonly,
       serverctrls=serverctrls,
       clientctrls=clientctrls,
@@ -752,14 +1024,20 @@ class SimpleLDAPObject:
       raise NO_UNIQUE_ENTRY('No or non-unique search result for %s' % (repr(filterstr)))
     return r[0]
 
-  def read_rootdse_s(self, filterstr='(objectClass=*)', attrlist=None):
+  def read_rootdse_s(self, filterstr=None, attrlist=None):
     """
     convenience wrapper around read_s() for reading rootDSE
     """
+    if self.bytes_mode:
+      base = b''
+      attrlist = attrlist or [b'*', b'+']
+    else:
+      base = u''
+      attrlist = attrlist or [u'*', u'+']
     ldap_rootdse = self.read_s(
-      '',
+      base,
       filterstr=filterstr,
-      attrlist=attrlist or ['*', '+'],
+      attrlist=attrlist,
     )
     return ldap_rootdse  # read_rootdse_s()
 
@@ -768,43 +1046,13 @@ class SimpleLDAPObject:
     returns all attribute values of namingContexts in rootDSE
     if namingContexts is not present (not readable) then empty list is returned
     """
+    if self.bytes_mode:
+      name = b'namingContexts'
+    else:
+      name = u'namingContexts'
     return self.read_rootdse_s(
-      attrlist=['namingContexts']
-    ).get('namingContexts', [])
-
-
-class NonblockingLDAPObject(SimpleLDAPObject):
-
-  def __init__(self,uri,trace_level=0,trace_file=None,result_timeout=-1):
-    self._result_timeout = result_timeout
-    SimpleLDAPObject.__init__(self,uri,trace_level,trace_file)
-
-  def result(self,msgid=ldap.RES_ANY,all=1,timeout=-1):
-    """
-    """
-    ldap_result = self._ldap_call(self._l.result,msgid,0,self._result_timeout)
-    if not all:
-      return ldap_result
-    start_time = time.time()
-    all_results = []
-    while all:
-      while ldap_result[0] is None:
-        if (timeout>=0) and (time.time()-start_time>timeout):
-          self._ldap_call(self._l.abandon,msgid)
-          raise ldap.TIMEOUT(
-            "LDAP time limit (%d secs) exceeded." % (timeout)
-          )
-        time.sleep(0.00001)
-        ldap_result = self._ldap_call(self._l.result,msgid,0,self._result_timeout)
-      if ldap_result[1] is None:
-        break
-      all_results.extend(ldap_result[1])
-      ldap_result = None,None
-    return all_results
-
-  def search_st(self,base,scope,filterstr='(objectClass=*)',attrlist=None,attrsonly=0,timeout=-1):
-    msgid = self.search(base,scope,filterstr,attrlist,attrsonly)
-    return self.result(msgid,all=1,timeout=timeout)
+      attrlist=[name]
+    ).get(name, [])
 
 
 class ReconnectLDAPObject(SimpleLDAPObject):
@@ -821,16 +1069,17 @@ class ReconnectLDAPObject(SimpleLDAPObject):
   """
 
   __transient_attrs__ = {
-    '_l':None,
-    '_ldap_object_lock':None,
-    '_trace_file':None,
-    '_reconnect_lock':None,
+    '_l',
+    '_ldap_object_lock',
+    '_trace_file',
+    '_reconnect_lock',
+    '_last_bind',
   }
 
   def __init__(
     self,uri,
-    trace_level=0,trace_file=None,trace_stack_limit=5,
-    retry_max=1,retry_delay=60.0
+    trace_level=0,trace_file=None,trace_stack_limit=5,bytes_mode=None,
+    bytes_strictness=None, retry_max=1, retry_delay=60.0
   ):
     """
     Parameters like SimpleLDAPObject.__init__() with these
@@ -844,24 +1093,34 @@ class ReconnectLDAPObject(SimpleLDAPObject):
     self._uri = uri
     self._options = []
     self._last_bind = None
-    SimpleLDAPObject.__init__(self,uri,trace_level,trace_file,trace_stack_limit)
+    SimpleLDAPObject.__init__(self, uri, trace_level, trace_file,
+                              trace_stack_limit, bytes_mode,
+                              bytes_strictness=bytes_strictness)
     self._reconnect_lock = ldap.LDAPLock(desc='reconnect lock within %s' % (repr(self)))
     self._retry_max = retry_max
     self._retry_delay = retry_delay
     self._start_tls = 0
-    self._reconnects_done = 0L
+    self._reconnects_done = 0
 
   def __getstate__(self):
     """return data representation for pickled object"""
-    d = {}
-    for k,v in self.__dict__.items():
-      if not self.__transient_attrs__.has_key(k):
-        d[k] = v
-    return d
+    state = {
+        k: v
+        for k,v in self.__dict__.items()
+        if k not in self.__transient_attrs__
+    }
+    state['_last_bind'] = self._last_bind[0].__name__, self._last_bind[1], self._last_bind[2]
+    return state
 
   def __setstate__(self,d):
     """set up the object from pickled data"""
+    hardfail = d.get('bytes_mode_hardfail')
+    if hardfail:
+        d.setdefault('bytes_strictness', 'error')
+    else:
+        d.setdefault('bytes_strictness', 'warn')
     self.__dict__.update(d)
+    self._last_bind = getattr(SimpleLDAPObject, self._last_bind[0]), self._last_bind[1], self._last_bind[2]
     self._ldap_object_lock = self._ldap_lock()
     self._reconnect_lock = ldap.LDAPLock(desc='reconnect lock within %s' % (repr(self)))
     self._trace_file = sys.stdout
@@ -876,7 +1135,7 @@ class ReconnectLDAPObject(SimpleLDAPObject):
       func(self,*args,**kwargs)
     else:
       # Send explicit anon simple bind request to provoke ldap.SERVER_DOWN in method reconnect()
-      SimpleLDAPObject.simple_bind_s(self,'','')
+      SimpleLDAPObject.simple_bind_s(self, None, None)
 
   def _restore_options(self):
     """Restore all recorded options"""
@@ -907,14 +1166,14 @@ class ReconnectLDAPObject(SimpleLDAPObject):
             SimpleLDAPObject.start_tls_s(self)
           # Repeat last simple or SASL bind
           self._apply_last_bind()
-        except (ldap.SERVER_DOWN,ldap.TIMEOUT),e:
+        except (ldap.SERVER_DOWN,ldap.TIMEOUT):
           if __debug__ and self._trace_level>=1:
             self._trace_file.write('*** %s reconnect to %s failed\n' % (
               counter_text,uri
             ))
           reconnect_counter = reconnect_counter-1
           if not reconnect_counter:
-            raise e
+            raise
           if __debug__ and self._trace_level>=1:
             self._trace_file.write('=> delay %s...\n' % (retry_delay))
           time.sleep(retry_delay)
@@ -924,14 +1183,14 @@ class ReconnectLDAPObject(SimpleLDAPObject):
             self._trace_file.write('*** %s reconnect to %s successful => repeat last operation\n' % (
               counter_text,uri
             ))
-          self._reconnects_done = self._reconnects_done + 1L
+          self._reconnects_done = self._reconnects_done + 1
           break
     finally:
       self._reconnect_lock.release()
     return # reconnect()
 
   def _apply_method_s(self,func,*args,**kwargs):
-    if not self.__dict__.has_key('_l'):
+    if not hasattr(self,'_l'):
       self.reconnect(self._uri,retry_max=self._retry_max,retry_delay=self._retry_delay)
     try:
       return func(self,*args,**kwargs)
