@@ -2,14 +2,9 @@
 ldapurl - handling of LDAP URLs as described in RFC 4516
 
 See https://www.python-ldap.org/ for details.
-
-Python compability note:
-This module only works with Python 2.0+ since
-1. string methods are used instead of module string and
-2. list comprehensions are used.
 """
 
-__version__ = '2.4.45'
+__version__ = '3.0.0'
 
 __all__ = [
   # constants
@@ -21,9 +16,7 @@ __all__ = [
   'LDAPUrlExtension','LDAPUrlExtensions','LDAPUrl'
 ]
 
-import UserDict
-
-from urllib import quote,unquote
+from ldap.compat import UserDict, quote, unquote
 
 LDAP_SCOPE_BASE = 0
 LDAP_SCOPE_ONELEVEL = 1
@@ -68,8 +61,7 @@ def ldapUrlEscape(s):
   """Returns URL encoding of string s"""
   return quote(s).replace(',','%2C').replace('/','%2F')
 
-
-class LDAPUrlExtension:
+class LDAPUrlExtension(object):
   """
   Class for parsing and unparsing LDAP URL extensions
   as described in RFC 4516.
@@ -138,14 +130,14 @@ class LDAPUrlExtension:
     return not self.__eq__(other)
 
 
-class LDAPUrlExtensions(UserDict.UserDict):
+class LDAPUrlExtensions(UserDict):
   """
   Models a collection of LDAP URL extensions as
   dictionary type
   """
 
   def __init__(self,default=None):
-    UserDict.UserDict.__init__(self)
+    UserDict.__init__(self)
     for k,v in (default or {}).items():
       self[k]=v
 
@@ -166,7 +158,7 @@ class LDAPUrlExtensions(UserDict.UserDict):
     ]
 
   def __str__(self):
-    return ','.join(map(str,self.values()))
+    return ','.join(str(v) for v in self.values())
 
   def __repr__(self):
     return '<%s.%s instance at %s: %s>' % (
@@ -192,7 +184,7 @@ class LDAPUrlExtensions(UserDict.UserDict):
     return ','.join([ v.unparse() for v in self.values() ])
 
 
-class LDAPUrl:
+class LDAPUrl(object):
   """
   Class for parsing and unparsing LDAP URLs
   as described in RFC 4516.
@@ -399,10 +391,10 @@ class LDAPUrl:
     )
 
   def __getattr__(self,name):
-    if self.attr2extype.has_key(name):
+    if name in self.attr2extype:
       extype = self.attr2extype[name]
       if self.extensions and \
-         self.extensions.has_key(extype) and \
+         extype in self.extensions and \
          not self.extensions[extype].exvalue is None:
         result = unquote(self.extensions[extype].exvalue)
       else:
@@ -414,7 +406,7 @@ class LDAPUrl:
     return result # __getattr__()
 
   def __setattr__(self,name,value):
-    if self.attr2extype.has_key(name):
+    if name in self.attr2extype:
       extype = self.attr2extype[name]
       if value is None:
         # A value of None means that extension is deleted
@@ -428,7 +420,7 @@ class LDAPUrl:
       self.__dict__[name] = value
 
   def __delattr__(self,name):
-    if self.attr2extype.has_key(name):
+    if name in self.attr2extype:
       extype = self.attr2extype[name]
       if self.extensions:
         try:
