@@ -5,7 +5,6 @@
 
 #include <math.h>
 #include <limits.h>
-#include "errors.h"
 #include "constants.h"
 #include "LDAPObject.h"
 #include "ldapcontrol.h"
@@ -14,18 +13,18 @@
 #include "options.h"
 
 #ifdef HAVE_SASL
-#include <sasl.h>
+#include <sasl/sasl.h>
 #endif
 
-static void free_attrs(char***, PyObject*);
+static void free_attrs(char***);
 
 /* constructor */
 
 LDAPObject*
-newLDAPObject( LDAP* l )
+newLDAPObject( LDAP* l ) 
 {
     LDAPObject* self = (LDAPObject*) PyObject_NEW(LDAPObject, &LDAP_Type);
-    if (self == NULL)
+    if (self == NULL) 
         return NULL;
     self->ldap = l;
     self->_save = NULL;
@@ -54,8 +53,8 @@ dealloc( LDAPObject* self )
  * utility functions
  */
 
-/*
- * check to see if the LDAPObject is valid,
+/* 
+ * check to see if the LDAPObject is valid, 
  * ie has been opened, and not closed. An exception is set if not valid.
  */
 
@@ -87,8 +86,8 @@ LDAPMod_DEL( LDAPMod* lm )
     PyMem_DEL(lm);
 }
 
-/*
- * convert a tuple of the form (int,str,[str,...])
+/* 
+ * convert a tuple of the form (int,str,[str,...]) 
  * or (str, [str,...]) if no_op is true, into an LDAPMod structure.
  * See ldap_modify(3) for details.
  *
@@ -100,7 +99,7 @@ LDAPMod_DEL( LDAPMod* lm )
 /* XXX - there is no way to pass complex-structured BER objects in here! */
 
 static LDAPMod*
-Tuple_to_LDAPMod( PyObject* tup, int no_op )
+Tuple_to_LDAPMod( PyObject* tup, int no_op ) 
 {
     int op;
     char *type;
@@ -109,17 +108,16 @@ Tuple_to_LDAPMod( PyObject* tup, int no_op )
     Py_ssize_t i, len, nstrs;
 
     if (!PyTuple_Check(tup)) {
-        PyErr_SetObject(PyExc_TypeError, Py_BuildValue("sO",
-           "expected a tuple", tup));
+        LDAPerror_TypeError("Tuple_to_LDAPMod(): expected a tuple", tup);
         return NULL;
     }
 
     if (no_op) {
-        if (!PyArg_ParseTuple( tup, "sO", &type, &list ))
+        if (!PyArg_ParseTuple( tup, "sO:Tuple_to_LDAPMod", &type, &list ))
                 return NULL;
         op = 0;
     } else {
-        if (!PyArg_ParseTuple( tup, "isO", &op, &type, &list ))
+        if (!PyArg_ParseTuple( tup, "isO:Tuple_to_LDAPMod", &op, &type, &list ))
                 return NULL;
     }
 
@@ -163,9 +161,7 @@ Tuple_to_LDAPMod( PyObject* tup, int no_op )
           if (item == NULL)
               goto error;
           if (!PyBytes_Check(item)) {
-              PyErr_SetObject( PyExc_TypeError, Py_BuildValue( "sO",
-                  "expected a byte-string in the list", item));
-              Py_DECREF(item);
+              LDAPerror_TypeError("Tuple_to_LDAPMod(): expected a byte string in the list", item);
               goto error;
           }
           lm->mod_bvalues[i]->bv_len = PyBytes_Size(item);
@@ -181,7 +177,7 @@ Tuple_to_LDAPMod( PyObject* tup, int no_op )
 nomem:
     PyErr_NoMemory();
 error:
-    if (lm)
+    if (lm) 
         LDAPMod_DEL(lm);
 
     return NULL;
@@ -197,8 +193,8 @@ LDAPMods_DEL( LDAPMod** lms ) {
     PyMem_DEL(lms);
 }
 
-/*
- * convert a list of tuples into a LDAPMod*[] array structure
+/* 
+ * convert a list of tuples into a LDAPMod*[] array structure 
  * NOTE: list of tuples must live longer than the LDAPMods
  */
 
@@ -210,27 +206,25 @@ List_to_LDAPMods( PyObject *list, int no_op ) {
     PyObject *item;
 
     if (!PySequence_Check(list)) {
-        PyErr_SetObject( PyExc_TypeError, Py_BuildValue("sO",
-                        "expected list of tuples", list ));
+        LDAPerror_TypeError("List_to_LDAPMods(): expected list of tuples", list);
         return NULL;
     }
 
     len = PySequence_Length(list);
 
     if (len < 0) {
-        PyErr_SetObject( PyExc_TypeError, Py_BuildValue("sO",
-                         "expected list of tuples", list ));
+        LDAPerror_TypeError("List_to_LDAPMods(): expected list of tuples", list);
         return NULL;
     }
 
     lms = PyMem_NEW(LDAPMod *, len + 1);
-    if (lms == NULL)
+    if (lms == NULL) 
         goto nomem;
 
     for (i = 0; i < len; i++) {
         lms[i] = NULL;
         item = PySequence_GetItem(list, i);
-        if (item == NULL)
+        if (item == NULL) 
             goto error;
         lms[i] = Tuple_to_LDAPMod(item, no_op);
         Py_DECREF(item);
@@ -254,24 +248,33 @@ error:
  */
 
 int
-attrs_from_List( PyObject *attrlist, char***attrsp, PyObject** seq) {
+attrs_from_List( PyObject *attrlist, char***attrsp) {
 
     char **attrs = NULL;
-    Py_ssize_t i, len;
-    PyObject *item;
-
-    *seq = NULL;
+    PyObject *seq = NULL;
 
     if (attrlist == Py_None) {
         /* None means a NULL attrlist */
+#if PY_MAJOR_VERSION == 2
     } else if (PyBytes_Check(attrlist)) {
+#else
+    } else if (PyUnicode_Check(attrlist)) {
+#endif
         /* caught by John Benninghoff <johnb@netscape.com> */
-        PyErr_SetObject( PyExc_TypeError, Py_BuildValue("sO",
-                  "expected *list* of strings, not a string", attrlist ));
+        LDAPerror_TypeError(
+            "attrs_from_List(): expected *list* of strings, not a string", attrlist);
         goto error;
     } else {
-        *seq = PySequence_Fast(attrlist, "expected list of strings or None");
-        if (*seq == NULL)
+        PyObject *item = NULL;
+        Py_ssize_t i, len, strlen;
+#if PY_MAJOR_VERSION >= 3
+        const char *str;
+#else
+        char *str;
+#endif
+
+        seq = PySequence_Fast(attrlist, "expected list of strings or None");
+        if (seq == NULL)
             goto error;
 
         len = PySequence_Length(attrlist);
@@ -282,17 +285,36 @@ attrs_from_List( PyObject *attrlist, char***attrsp, PyObject** seq) {
 
         for (i = 0; i < len; i++) {
             attrs[i] = NULL;
-            item = PySequence_Fast_GET_ITEM(*seq, i);
+            item = PySequence_Fast_GET_ITEM(seq, i);
             if (item == NULL)
                 goto error;
+#if PY_MAJOR_VERSION == 2
+            /* Encoded in Python to UTF-8 */
             if (!PyBytes_Check(item)) {
-                PyErr_SetObject(PyExc_TypeError, Py_BuildValue("sO",
-                                "expected string in list", item));
+                LDAPerror_TypeError("attrs_from_List(): expected bytes in list", item);
                 goto error;
             }
-            attrs[i] = PyBytes_AsString(item);
+            if (PyBytes_AsStringAndSize(item, &str, &strlen) == -1) {
+                goto error;
+            }
+#else
+            if (!PyUnicode_Check(item)) {
+                LDAPerror_TypeError("attrs_from_List(): expected string in list", item);
+                goto error;
+            }
+            str = PyUnicode_AsUTF8AndSize(item, &strlen);
+#endif
+            /* Make a copy. PyBytes_AsString* / PyUnicode_AsUTF8* return
+             * internal values that must be treated like const char. Python
+             * 3.7 actually returns a const char.
+             */
+            attrs[i] = (char *)PyMem_NEW(char *, strlen + 1);
+            if (attrs[i] == NULL)
+                goto nomem;
+            memcpy(attrs[i], str, strlen + 1);
         }
         attrs[len] = NULL;
+        Py_DECREF(seq);
     }
 
     *attrsp = attrs;
@@ -301,22 +323,26 @@ attrs_from_List( PyObject *attrlist, char***attrsp, PyObject** seq) {
 nomem:
     PyErr_NoMemory();
 error:
-    free_attrs(&attrs, *seq);
+    Py_XDECREF(seq);
+    free_attrs(&attrs);
     return 0;
 }
 
 /* free memory allocated from above routine */
 
 static void
-free_attrs( char*** attrsp, PyObject* seq ) {
+free_attrs( char*** attrsp) {
     char **attrs = *attrsp;
+    char **p;
 
-    if (attrs != NULL) {
-        PyMem_DEL(attrs);
-        *attrsp = NULL;
+    if (attrs == NULL)
+        return;
+
+    *attrsp = NULL;
+    for (p = attrs; *p != NULL; p++) {
+        PyMem_DEL(*p);
     }
-
-    Py_XDECREF(seq);
+    PyMem_DEL(attrs);
 }
 
 /*------------------------------------------------------------
@@ -335,7 +361,7 @@ l_ldap_unbind_ext( LDAPObject* self, PyObject* args )
 
     int ldaperror;
 
-    if (!PyArg_ParseTuple( args, "|OO", &serverctrls, &clientctrls)) return NULL;
+    if (!PyArg_ParseTuple( args, "|OO:unbind_ext", &serverctrls, &clientctrls)) return NULL;
     if (not_valid(self)) return NULL;
 
     if (!PyNone_Check(serverctrls)) {
@@ -344,8 +370,10 @@ l_ldap_unbind_ext( LDAPObject* self, PyObject* args )
     }
 
     if (!PyNone_Check(clientctrls)) {
-        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs)) {
+            LDAPControl_List_DEL( server_ldcs );
             return NULL;
+        }
     }
 
     LDAP_BEGIN_ALLOW_THREADS( self );
@@ -376,7 +404,7 @@ l_ldap_abandon_ext( LDAPObject* self, PyObject* args )
 
     int ldaperror;
 
-    if (!PyArg_ParseTuple( args, "i|OO", &msgid, &serverctrls, &clientctrls)) return NULL;
+    if (!PyArg_ParseTuple( args, "i|OO:abandon_ext", &msgid, &serverctrls, &clientctrls)) return NULL;
     if (not_valid(self)) return NULL;
 
     if (!PyNone_Check(serverctrls)) {
@@ -385,8 +413,10 @@ l_ldap_abandon_ext( LDAPObject* self, PyObject* args )
     }
 
     if (!PyNone_Check(clientctrls)) {
-        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs)) {
+            LDAPControl_List_DEL( server_ldcs );
             return NULL;
+        }
     }
 
     LDAP_BEGIN_ALLOW_THREADS( self );
@@ -419,7 +449,7 @@ l_ldap_add_ext( LDAPObject* self, PyObject *args )
     int ldaperror;
     LDAPMod **mods;
 
-    if (!PyArg_ParseTuple( args, "sO|OO", &dn, &modlist, &serverctrls, &clientctrls )) return NULL;
+    if (!PyArg_ParseTuple( args, "sO|OO:add_ext", &dn, &modlist, &serverctrls, &clientctrls )) return NULL;
     if (not_valid(self)) return NULL;
 
     mods = List_to_LDAPMods( modlist, 1 );
@@ -427,13 +457,18 @@ l_ldap_add_ext( LDAPObject* self, PyObject *args )
         return NULL;
 
     if (!PyNone_Check(serverctrls)) {
-        if (!LDAPControls_from_object(serverctrls, &server_ldcs))
+        if (!LDAPControls_from_object(serverctrls, &server_ldcs)) {
+            LDAPMods_DEL( mods );
             return NULL;
+        }
     }
 
     if (!PyNone_Check(clientctrls)) {
-        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs)) {
+            LDAPMods_DEL( mods );
+            LDAPControl_List_DEL( server_ldcs );
             return NULL;
+        }
     }
 
     LDAP_BEGIN_ALLOW_THREADS( self );
@@ -464,7 +499,7 @@ l_ldap_simple_bind( LDAPObject* self, PyObject* args )
     LDAPControl** client_ldcs = NULL;
     struct berval cred;
 
-    if (!PyArg_ParseTuple( args, "ss#|OO", &who, &cred.bv_val, &cred_len, &serverctrls, &clientctrls )) return NULL;
+    if (!PyArg_ParseTuple( args, "zz#|OO:simple_bind", &who, &cred.bv_val, &cred_len, &serverctrls, &clientctrls )) return NULL;
     cred.bv_len = (ber_len_t) cred_len;
 
     if (not_valid(self)) return NULL;
@@ -475,8 +510,10 @@ l_ldap_simple_bind( LDAPObject* self, PyObject* args )
     }
 
     if (!PyNone_Check(clientctrls)) {
-        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs)) {
+            LDAPControl_List_DEL( server_ldcs );
             return NULL;
+        }
     }
 
     LDAP_BEGIN_ALLOW_THREADS( self );
@@ -509,7 +546,7 @@ l_ldap_simple_bind( LDAPObject* self, PyObject* args )
      auth modules ("mechanisms"), or try
 
        ldapsearch   -b "" -s base -LLL -x  supportedSASLMechanisms
-
+     
      (perhaps with an additional -h and -p argument for ldap host and
      port). The latter will show you which SASL mechanisms are known
      to the LDAP server. If you do not want to set up Kerberos, you
@@ -529,7 +566,7 @@ l_ldap_simple_bind( LDAPObject* self, PyObject* args )
      argument specifies, which information should be passed back to
      the SASL lib (see SASL_CB_xxx in sasl.h)
 */
-static int interaction ( unsigned flags,
+static int interaction ( unsigned flags, 
                          sasl_interact_t *interact,
                          PyObject* SASLObject )
 {
@@ -541,14 +578,14 @@ static int interaction ( unsigned flags,
                                "isss",
                                interact->id,  /* see sasl.h */
                                interact->challenge,
-                               interact->prompt,
+                               interact->prompt,   
                                interact->defresult);
 
-  if (result == NULL)
+  if (result == NULL) 
     /*searching for a better error code */
-    return LDAP_OPERATIONS_ERROR;
+    return LDAP_OPERATIONS_ERROR; 
   c_result = PyBytes_AsString(result); /*xxx Error checking?? */
-
+  
   /* according to the sasl docs, we should malloc() the returned
      string only for calls where interact->id == SASL_CB_PASS, so we
      probably leak a few bytes per ldap bind. However, if I restrict
@@ -562,15 +599,15 @@ static int interaction ( unsigned flags,
   /* We _should_ overwrite the python string buffer for security
      reasons, however we may not (api/stringObjects.html). Any ideas?
   */
-
+  
   Py_DECREF(result); /*not needed any longer */
   result = NULL;
-
+  
   return LDAP_SUCCESS;
 }
 
 
-/*
+/* 
   This function will be called by ldap_sasl_interactive_bind(). The
   "*in" is an array of sasl_interact_t's (see sasl.h for a
   reference). The last interact in the array has an interact->id of
@@ -578,8 +615,8 @@ static int interaction ( unsigned flags,
 
 */
 
-int py_ldap_sasl_interaction(   LDAP *ld,
-                                unsigned flags,
+int py_ldap_sasl_interaction(   LDAP *ld, 
+                                unsigned flags, 
                                 void *defaults,
                                 void *in )
 {
@@ -612,7 +649,7 @@ l_ldap_sasl_bind_s( LDAPObject* self, PyObject* args )
     struct berval *servercred;
     int ldaperror;
 
-    if (!PyArg_ParseTuple(args, "zzz#OO", &dn, &mechanism, &cred.bv_val, &cred_len, &serverctrls, &clientctrls ))
+    if (!PyArg_ParseTuple(args, "zzz#OO:sasl_bind_s", &dn, &mechanism, &cred.bv_val, &cred_len, &serverctrls, &clientctrls ))
         return NULL;
 
     if (not_valid(self)) return NULL;
@@ -624,8 +661,10 @@ l_ldap_sasl_bind_s( LDAPObject* self, PyObject* args )
             return NULL;
     }
     if (!PyNone_Check(clientctrls)) {
-        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs)) {
+            LDAPControl_List_DEL( server_ldcs );
             return NULL;
+        }
     }
 
     LDAP_BEGIN_ALLOW_THREADS( self );
@@ -633,7 +672,7 @@ l_ldap_sasl_bind_s( LDAPObject* self, PyObject* args )
                                  dn,
                                  mechanism,
                                  cred.bv_val ? &cred : NULL,
-                                 (LDAPControl**) server_ldcs,
+                                 (LDAPControl**) server_ldcs, 
                                  (LDAPControl**) client_ldcs,
                                  &servercred);
     LDAP_END_ALLOW_THREADS( self );
@@ -649,7 +688,7 @@ l_ldap_sasl_bind_s( LDAPObject* self, PyObject* args )
     return PyInt_FromLong( ldaperror );
 }
 
-static PyObject*
+static PyObject* 
 l_ldap_sasl_interactive_bind_s( LDAPObject* self, PyObject* args )
 {
     char *c_mechanism;
@@ -666,17 +705,17 @@ l_ldap_sasl_interactive_bind_s( LDAPObject* self, PyObject* args )
 
     static unsigned sasl_flags = LDAP_SASL_QUIET;
 
-    /*
+    /* 
      * In Python 2.3+, a "I" format argument indicates that we're either converting
      * the Python object into a long or an unsigned int. In versions prior to that,
      * it will always convert to a long. Since the sasl_flags variable is an
      * unsigned int, we need to use the "I" flag if we're running Python 2.3+ and a
-     * "i" otherwise.
+     * "i" otherwise. 
      */
 #if (PY_MAJOR_VERSION == 2) && (PY_MINOR_VERSION < 3)
-    if (!PyArg_ParseTuple(args, "sOOOi", &who, &SASLObject, &serverctrls, &clientctrls, &sasl_flags ))
+    if (!PyArg_ParseTuple(args, "sOOOi:sasl_interactive_bind_s", &who, &SASLObject, &serverctrls, &clientctrls, &sasl_flags ))
 #else
-    if (!PyArg_ParseTuple(args, "sOOOI", &who, &SASLObject, &serverctrls, &clientctrls, &sasl_flags ))
+    if (!PyArg_ParseTuple(args, "sOOOI:sasl_interactive_bind_s", &who, &SASLObject, &serverctrls, &clientctrls, &sasl_flags ))
 #endif
       return NULL;
 
@@ -688,8 +727,10 @@ l_ldap_sasl_interactive_bind_s( LDAPObject* self, PyObject* args )
     }
 
     if (!PyNone_Check(clientctrls)) {
-        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs)) {
+            LDAPControl_List_DEL( server_ldcs );
             return NULL;
+        }
     }
 
     /* now we extract the sasl mechanism from the SASL Object */
@@ -704,13 +745,13 @@ l_ldap_sasl_interactive_bind_s( LDAPObject* self, PyObject* args )
        Python object SASLObject, but passing it through some
        static variable would destroy thread safety, IMHO.
      */
-    msgid = ldap_sasl_interactive_bind_s(self->ldap,
-                                         who,
-                                         c_mechanism,
-                                         (LDAPControl**) server_ldcs,
+    msgid = ldap_sasl_interactive_bind_s(self->ldap, 
+                                         who, 
+                                         c_mechanism, 
+                                         (LDAPControl**) server_ldcs, 
                                          (LDAPControl**) client_ldcs,
-                                         sasl_flags,
-                                         py_ldap_sasl_interaction,
+                                         sasl_flags, 
+                                         py_ldap_sasl_interaction, 
                                          SASLObject);
 
     LDAPControl_List_DEL( server_ldcs );
@@ -739,7 +780,7 @@ l_ldap_cancel( LDAPObject* self, PyObject* args )
 
     int ldaperror;
 
-    if (!PyArg_ParseTuple( args, "i|OO", &cancelid, &serverctrls, &clientctrls)) return NULL;
+    if (!PyArg_ParseTuple( args, "i|OO:cancel", &cancelid, &serverctrls, &clientctrls)) return NULL;
     if (not_valid(self)) return NULL;
 
     if (!PyNone_Check(serverctrls)) {
@@ -748,8 +789,10 @@ l_ldap_cancel( LDAPObject* self, PyObject* args )
     }
 
     if (!PyNone_Check(clientctrls)) {
-        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs)) {
+            LDAPControl_List_DEL( server_ldcs );
             return NULL;
+        }
     }
 
     LDAP_BEGIN_ALLOW_THREADS( self );
@@ -783,7 +826,7 @@ l_ldap_compare_ext( LDAPObject* self, PyObject *args )
     Py_ssize_t value_len;
     struct berval value;
 
-    if (!PyArg_ParseTuple( args, "sss#|OO", &dn, &attr, &value.bv_val, &value_len, &serverctrls, &clientctrls )) return NULL;
+    if (!PyArg_ParseTuple( args, "sss#|OO:compare_ext", &dn, &attr, &value.bv_val, &value_len, &serverctrls, &clientctrls )) return NULL;
     value.bv_len = (ber_len_t) value_len;
 
     if (not_valid(self)) return NULL;
@@ -794,8 +837,10 @@ l_ldap_compare_ext( LDAPObject* self, PyObject *args )
     }
 
     if (!PyNone_Check(clientctrls)) {
-        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs)) {
+            LDAPControl_List_DEL( server_ldcs );
             return NULL;
+        }
     }
 
     LDAP_BEGIN_ALLOW_THREADS( self );
@@ -826,7 +871,7 @@ l_ldap_delete_ext( LDAPObject* self, PyObject *args )
     int msgid;
     int ldaperror;
 
-    if (!PyArg_ParseTuple( args, "s|OO", &dn, &serverctrls, &clientctrls )) return NULL;
+    if (!PyArg_ParseTuple( args, "s|OO:delete_ext", &dn, &serverctrls, &clientctrls )) return NULL;
     if (not_valid(self)) return NULL;
 
     if (!PyNone_Check(serverctrls)) {
@@ -835,8 +880,10 @@ l_ldap_delete_ext( LDAPObject* self, PyObject *args )
     }
 
     if (!PyNone_Check(clientctrls)) {
-        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs)) {
+            LDAPControl_List_DEL( server_ldcs );
             return NULL;
+        }
     }
 
     LDAP_BEGIN_ALLOW_THREADS( self );
@@ -869,7 +916,7 @@ l_ldap_modify_ext( LDAPObject* self, PyObject *args )
     int ldaperror;
     LDAPMod **mods;
 
-    if (!PyArg_ParseTuple( args, "sO|OO", &dn, &modlist, &serverctrls, &clientctrls )) return NULL;
+    if (!PyArg_ParseTuple( args, "sO|OO:modify_ext", &dn, &modlist, &serverctrls, &clientctrls )) return NULL;
     if (not_valid(self)) return NULL;
 
     mods = List_to_LDAPMods( modlist, 0 );
@@ -877,13 +924,18 @@ l_ldap_modify_ext( LDAPObject* self, PyObject *args )
         return NULL;
 
     if (!PyNone_Check(serverctrls)) {
-        if (!LDAPControls_from_object(serverctrls, &server_ldcs))
+        if (!LDAPControls_from_object(serverctrls, &server_ldcs)) {
+            LDAPMods_DEL( mods );
             return NULL;
+        }
     }
 
     if (!PyNone_Check(clientctrls)) {
-        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs)) {
+            LDAPMods_DEL( mods );
+            LDAPControl_List_DEL( server_ldcs );
             return NULL;
+        }
     }
 
     LDAP_BEGIN_ALLOW_THREADS( self );
@@ -917,7 +969,7 @@ l_ldap_rename( LDAPObject* self, PyObject *args )
     int msgid;
     int ldaperror;
 
-    if (!PyArg_ParseTuple( args, "ss|ziOO", &dn, &newrdn, &newSuperior, &delold, &serverctrls, &clientctrls ))
+    if (!PyArg_ParseTuple( args, "ss|ziOO:rename", &dn, &newrdn, &newSuperior, &delold, &serverctrls, &clientctrls ))
         return NULL;
     if (not_valid(self)) return NULL;
 
@@ -927,8 +979,10 @@ l_ldap_rename( LDAPObject* self, PyObject *args )
     }
 
     if (!PyNone_Check(clientctrls)) {
-        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs)) {
+            LDAPControl_List_DEL( server_ldcs );
             return NULL;
+        }
     }
 
     LDAP_BEGIN_ALLOW_THREADS( self );
@@ -963,12 +1017,15 @@ l_ldap_result4( LDAPObject* self, PyObject *args )
     PyObject *result_str, *retval, *pmsg, *pyctrls = 0;
     int res_msgid = 0;
     char *retoid = 0;
-    PyObject *valuestr = 0;
+    PyObject *valuestr = NULL;
+    int result = LDAP_SUCCESS;
+    char **refs = NULL;
+    LDAPControl **serverctrls = 0;
 
-    if (!PyArg_ParseTuple( args, "|iidiii", &msgid, &all, &timeout, &add_ctrls, &add_intermediates, &add_extop ))
+    if (!PyArg_ParseTuple( args, "|iidiii:result4", &msgid, &all, &timeout, &add_ctrls, &add_intermediates, &add_extop ))
         return NULL;
     if (not_valid(self)) return NULL;
-
+    
     if (timeout >= 0) {
         tvp = &tv;
         set_timeval_from_double( tvp, timeout );
@@ -999,9 +1056,6 @@ l_ldap_result4( LDAPObject* self, PyObject *args )
     if (msg)
         res_msgid = ldap_msgid(msg);
 
-    int result = LDAP_SUCCESS;
-    char **refs = NULL;
-    LDAPControl **serverctrls = 0;
     if (res_type == LDAP_RES_SEARCH_ENTRY) {
         /* LDAPmessage_to_python will parse entries and read the controls for each entry */
     } else if (res_type == LDAP_RES_SEARCH_REFERENCE) {
@@ -1021,7 +1075,7 @@ l_ldap_result4( LDAPObject* self, PyObject *args )
             }
             ber_bvfree( retdata );
         }
-
+            
         LDAP_BEGIN_ALLOW_THREADS( self );
         rc = ldap_parse_result( self->ldap, msg, &result, NULL, NULL, &refs,
                                 &serverctrls, 0 );
@@ -1036,6 +1090,7 @@ l_ldap_result4( LDAPObject* self, PyObject *args )
         } else
             e = "ldap_parse_result";
         ldap_msgfree(msg);
+        Py_XDECREF(valuestr);
         return LDAPerror( self->ldap, e );
     }
 
@@ -1045,13 +1100,19 @@ l_ldap_result4( LDAPObject* self, PyObject *args )
         ldap_set_option(self->ldap, LDAP_OPT_ERROR_NUMBER, &err);
         LDAP_END_ALLOW_THREADS( self );
         ldap_msgfree(msg);
+        Py_XDECREF(valuestr);
         return LDAPerror(self->ldap, "LDAPControls_to_List");
     }
     ldap_controls_free(serverctrls);
 
     pmsg = LDAPmessage_to_python( self->ldap, msg, add_ctrls, add_intermediates );
 
-    result_str = LDAPconstant( res_type );
+    if (res_type == 0) {
+        result_str = Py_None;
+        Py_INCREF(Py_None);
+    } else {
+        result_str = PyInt_FromLong( res_type );
+    }
 
     if (pmsg == NULL) {
             retval = NULL;
@@ -1068,9 +1129,7 @@ l_ldap_result4( LDAPObject* self, PyObject *args )
         Py_DECREF(pmsg);
         }
     }
-    if (valuestr) {
-        Py_DECREF(valuestr);
-    }
+    Py_XDECREF(valuestr);
     Py_XDECREF(pyctrls);
     Py_DECREF(result_str);
     return retval;
@@ -1091,7 +1150,6 @@ l_ldap_search_ext( LDAPObject* self, PyObject* args )
 
     PyObject *serverctrls = Py_None;
     PyObject *clientctrls = Py_None;
-    PyObject *attrs_seq = NULL;
     LDAPControl** server_ldcs = NULL;
     LDAPControl** client_ldcs = NULL;
 
@@ -1104,12 +1162,12 @@ l_ldap_search_ext( LDAPObject* self, PyObject* args )
     int msgid;
     int ldaperror;
 
-    if (!PyArg_ParseTuple( args, "sis|OiOOdi",
+    if (!PyArg_ParseTuple( args, "sis|OiOOdi:search_ext",
                            &base, &scope, &filter, &attrlist, &attrsonly,
                            &serverctrls, &clientctrls, &timeout, &sizelimit )) return NULL;
     if (not_valid(self)) return NULL;
 
-    if (!attrs_from_List( attrlist, &attrs, &attrs_seq ))
+    if (!attrs_from_List( attrlist, &attrs ))
          return NULL;
 
     if (timeout >= 0) {
@@ -1120,13 +1178,18 @@ l_ldap_search_ext( LDAPObject* self, PyObject* args )
     }
 
     if (!PyNone_Check(serverctrls)) {
-        if (!LDAPControls_from_object(serverctrls, &server_ldcs))
+        if (!LDAPControls_from_object(serverctrls, &server_ldcs)) {
+            free_attrs( &attrs );
             return NULL;
+        }
     }
 
     if (!PyNone_Check(clientctrls)) {
-        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs)) {
+            free_attrs( &attrs );
+            LDAPControl_List_DEL( server_ldcs );
             return NULL;
+        }
     }
 
     LDAP_BEGIN_ALLOW_THREADS( self );
@@ -1134,7 +1197,7 @@ l_ldap_search_ext( LDAPObject* self, PyObject* args )
                              server_ldcs, client_ldcs, tvp, sizelimit, &msgid );
     LDAP_END_ALLOW_THREADS( self );
 
-    free_attrs( &attrs,  attrs_seq);
+    free_attrs( &attrs );
     LDAPControl_List_DEL( server_ldcs );
     LDAPControl_List_DEL( client_ldcs );
 
@@ -1142,7 +1205,7 @@ l_ldap_search_ext( LDAPObject* self, PyObject* args )
         return LDAPerror( self->ldap, "ldap_search_ext" );
 
     return PyInt_FromLong( msgid );
-}
+}       
 
 
 /* ldap_whoami_s (available since OpenLDAP 2.1.13) */
@@ -1161,7 +1224,7 @@ l_ldap_whoami_s( LDAPObject* self, PyObject* args )
 
     int ldaperror;
 
-    if (!PyArg_ParseTuple( args, "|OO", &serverctrls, &clientctrls)) return NULL;
+    if (!PyArg_ParseTuple( args, "|OO:whoami_s", &serverctrls, &clientctrls)) return NULL;
     if (not_valid(self)) return NULL;
 
     if (!PyNone_Check(serverctrls)) {
@@ -1170,8 +1233,10 @@ l_ldap_whoami_s( LDAPObject* self, PyObject* args )
     }
 
     if (!PyNone_Check(clientctrls)) {
-        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs)) {
+            LDAPControl_List_DEL( server_ldcs );
             return NULL;
+        }
     }
 
     LDAP_BEGIN_ALLOW_THREADS( self );
@@ -1181,10 +1246,13 @@ l_ldap_whoami_s( LDAPObject* self, PyObject* args )
     LDAPControl_List_DEL( server_ldcs );
     LDAPControl_List_DEL( client_ldcs );
 
-    if ( ldaperror!=LDAP_SUCCESS )
+    if ( ldaperror!=LDAP_SUCCESS ) {
+        ber_bvfree(bvalue);
         return LDAPerror( self->ldap, "ldap_whoami_s" );
+    }
 
-    result = LDAPberval_to_object(bvalue);
+    result = LDAPberval_to_unicode_object(bvalue);
+    ber_bvfree(bvalue);
 
     return result;
 }
@@ -1197,7 +1265,7 @@ l_ldap_start_tls_s( LDAPObject* self, PyObject* args )
 {
     int ldaperror;
 
-    if (!PyArg_ParseTuple( args, "" )) return NULL;
+    if (!PyArg_ParseTuple( args, ":start_tls_s" )) return NULL;
     if (not_valid(self)) return NULL;
 
     LDAP_BEGIN_ALLOW_THREADS( self );
@@ -1224,7 +1292,7 @@ l_ldap_set_option(PyObject* self, PyObject *args)
 
     if (!PyArg_ParseTuple(args, "iO:set_option", &option, &value))
         return NULL;
-    if (LDAP_set_option((LDAPObject *)self, option, value) == -1)
+    if (!LDAP_set_option((LDAPObject *)self, option, value))
         return NULL;
     Py_INCREF(Py_None);
     return Py_None;
@@ -1263,13 +1331,13 @@ l_ldap_passwd( LDAPObject* self, PyObject *args )
     int msgid;
     int ldaperror;
 
-    if (!PyArg_ParseTuple( args, "z#z#z#|OO", &user.bv_val, &user_len, &oldpw.bv_val, &oldpw_len, &newpw.bv_val, &newpw_len, &serverctrls, &clientctrls ))
+    if (!PyArg_ParseTuple( args, "z#z#z#|OO:passwd", &user.bv_val, &user_len, &oldpw.bv_val, &oldpw_len, &newpw.bv_val, &newpw_len, &serverctrls, &clientctrls ))
         return NULL;
 
     user.bv_len = (ber_len_t) user_len;
     oldpw.bv_len = (ber_len_t) oldpw_len;
     newpw.bv_len = (ber_len_t) newpw_len;
-
+    
     if (not_valid(self)) return NULL;
 
     if (!PyNone_Check(serverctrls)) {
@@ -1278,8 +1346,10 @@ l_ldap_passwd( LDAPObject* self, PyObject *args )
     }
 
     if (!PyNone_Check(clientctrls)) {
-        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs)) {
+            LDAPControl_List_DEL( server_ldcs );
             return NULL;
+        }
     }
 
     LDAP_BEGIN_ALLOW_THREADS( self );
@@ -1291,7 +1361,7 @@ l_ldap_passwd( LDAPObject* self, PyObject *args )
             client_ldcs,
             &msgid );
     LDAP_END_ALLOW_THREADS( self );
-
+    
     LDAPControl_List_DEL( server_ldcs );
     LDAPControl_List_DEL( client_ldcs );
 
@@ -1317,7 +1387,7 @@ l_ldap_extended_operation( LDAPObject* self, PyObject *args )
     int msgid;
     int ldaperror;
 
-    if (!PyArg_ParseTuple( args, "sz#|OO", &reqoid, &reqvalue.bv_val, &reqvalue.bv_len, &serverctrls, &clientctrls ))
+    if (!PyArg_ParseTuple( args, "sz#|OO:extended_operation", &reqoid, &reqvalue.bv_val, &reqvalue.bv_len, &serverctrls, &clientctrls ))
         return NULL;
 
     if (not_valid(self)) return NULL;
@@ -1328,8 +1398,10 @@ l_ldap_extended_operation( LDAPObject* self, PyObject *args )
     }
 
     if (!PyNone_Check(clientctrls)) {
-        if (!LDAPControls_from_object(clientctrls, &client_ldcs))
+        if (!LDAPControls_from_object(clientctrls, &client_ldcs)) {
+            LDAPControl_List_DEL( server_ldcs );
             return NULL;
+        }
     }
 
     LDAP_BEGIN_ALLOW_THREADS( self );
@@ -1339,7 +1411,7 @@ l_ldap_extended_operation( LDAPObject* self, PyObject *args )
             client_ldcs,
             &msgid );
     LDAP_END_ALLOW_THREADS( self );
-
+    
     LDAPControl_List_DEL( server_ldcs );
     LDAPControl_List_DEL( client_ldcs );
 
@@ -1380,40 +1452,18 @@ static PyMethodDef methods[] = {
     { NULL, NULL }
 };
 
-/* get attribute */
-
-static PyObject*
-getattr(LDAPObject* self, char* name)
-{
-        return Py_FindMethod(methods, (PyObject*)self, name);
-}
-
-/* set attribute */
-
-static int
-setattr(LDAPObject* self, char* name, PyObject* value)
-{
-        PyErr_SetString(PyExc_AttributeError, name);
-        return -1;
-}
-
 /* type entry */
 
 PyTypeObject LDAP_Type = {
-#if defined(MS_WINDOWS) || defined(__CYGWIN__)
-        /* see http://www.python.org/doc/FAQ.html#3.24 */
         PyVarObject_HEAD_INIT(NULL, 0)
-#else /* ! MS_WINDOWS */
-        PyVarObject_HEAD_INIT(&PyType_Type, 0)
-#endif /* MS_WINDOWS */
         "LDAP",                 /*tp_name*/
         sizeof(LDAPObject),     /*tp_basicsize*/
         0,                      /*tp_itemsize*/
         /* methods */
         (destructor)dealloc,    /*tp_dealloc*/
         0,                      /*tp_print*/
-        (getattrfunc)getattr,   /*tp_getattr*/
-        (setattrfunc)setattr,   /*tp_setattr*/
+        0,                      /*tp_getattr*/
+        0,                      /*tp_setattr*/
         0,                      /*tp_compare*/
         0,                      /*tp_repr*/
         0,                      /*tp_as_number*/
