@@ -1,8 +1,8 @@
 /* common utility macros
  * See https://www.python-ldap.org/ for details. */
 
-#ifndef __h_common
-#define __h_common
+#ifndef __h_common 
+#define __h_common 
 
 #define PY_SSIZE_T_CLEAN
 
@@ -24,11 +24,16 @@
 #define streq( a, b ) \
 	( (*(a)==*(b)) && 0==strcmp(a,b) )
 
+extern PyObject* LDAPerror_TypeError(const char *, PyObject *);
+
 void LDAPadd_methods( PyObject*d, PyMethodDef*methods );
 #define PyNone_Check(o) ((o) == Py_None)
 
-#if PY_MAJOR_VERSION >= 3
+/* Py2/3 compatibility */
+#if PY_VERSION_HEX >= 0x03000000
+/* In Python 3, alias PyInt to PyLong */
 #define PyInt_FromLong PyLong_FromLong
 #endif
 
 #endif /* __h_common_ */
+
