@@ -1,13 +1,19 @@
-/* See https://www.python-ldap.org/ for details.
- * $Id: constants.h,v 1.7 2017/08/15 16:21:59 stroeder Exp $ */
+/* See https://www.python-ldap.org/ for details. */
 
 #ifndef __h_constants_
 #define __h_constants_
 
 #include "common.h"
-extern void LDAPinit_constants( PyObject* d );
+#include "lber.h"
+#include "ldap.h"
+
+extern int LDAPinit_constants( PyObject* m );
 extern PyObject* LDAPconstant( int );
 
+extern PyObject* LDAPexception_class;
+extern PyObject* LDAPerror( LDAP*, char*msg );
+PyObject* LDAPerr(int errnum);
+
 #ifndef LDAP_CONTROL_PAGE_OID
 #define LDAP_CONTROL_PAGE_OID "1.2.840.113556.1.4.319"
 #endif /* !LDAP_CONTROL_PAGE_OID */
