@@ -4,7 +4,7 @@
 #include "functions.h"
 #include "LDAPObject.h"
 #include "berval.h"
-#include "errors.h"
+#include "constants.h"
 #include "options.h"
 
 /* ldap_initialize */
@@ -16,7 +16,7 @@ l_ldap_initialize(PyObject* unused, PyObject *args)
     LDAP *ld = NULL;
     int ret;
 
-    if (!PyArg_ParseTuple(args, "s", &uri))
+    if (!PyArg_ParseTuple(args, "s:initialize", &uri))
     	return NULL;
 
     Py_BEGIN_ALLOW_THREADS
@@ -46,8 +46,8 @@ l_ldap_str2dn( PyObject* unused, PyObject *args )
      * ((('a','b',1),('c','d',1)),(('e','f',1),))
      * The integers are a bit combination of the AVA_* flags
      */
-    if (!PyArg_ParseTuple( args, "z#|i:str2dn",
-	    &str.bv_val, &str_len, &flags ))
+    if (!PyArg_ParseTuple( args, "z#|i:str2dn", 
+	    &str.bv_val, &str_len, &flags )) 
 	return NULL;
     str.bv_len = (ber_len_t) str_len;
 
@@ -77,8 +77,8 @@ l_ldap_str2dn( PyObject* unused, PyObject *args )
 	    PyObject *tuple;
 
 	    tuple = Py_BuildValue("(O&O&i)",
-		LDAPberval_to_object, &ava->la_attr,
-		LDAPberval_to_object, &ava->la_value,
+		LDAPberval_to_unicode_object, &ava->la_attr,
+		LDAPberval_to_unicode_object, &ava->la_value,
 		ava->la_flags & ~(LDAP_AVA_FREE_ATTR|LDAP_AVA_FREE_VALUE));
 	    if (!tuple) {
 		Py_DECREF(rdnlist);
