@@ -4,7 +4,7 @@
 #include "LDAPObject.h"
 #include "ldapcontrol.h"
 #include "berval.h"
-#include "errors.h"
+#include "constants.h"
 
 #include "lber.h"
 
@@ -27,13 +27,13 @@ LDAPControl_DumpList( LDAPControl** lcs ) {
 } */
 
 /* Free a single LDAPControl object created by Tuple_to_LDAPControl */
-
+  
 static void
 LDAPControl_DEL( LDAPControl* lc )
 {
     if (lc == NULL)
         return;
-
+  
     if (lc->ldctl_oid)
         PyMem_DEL(lc->ldctl_oid);
     PyMem_DEL(lc);
@@ -72,14 +72,13 @@ Tuple_to_LDAPControl( PyObject* tup )
     Py_ssize_t len;
 
     if (!PyTuple_Check(tup)) {
-	PyErr_SetObject(PyExc_TypeError, Py_BuildValue("sO",
-	   "expected a tuple", tup));
-	return NULL;
+      LDAPerror_TypeError("Tuple_to_LDAPControl(): expected a tuple", tup);
+      return NULL;
     }
 
-    if (!PyArg_ParseTuple( tup, "sbO", &oid, &iscritical, &bytes ))
+    if (!PyArg_ParseTuple( tup, "sbO:Tuple_to_LDAPControl", &oid, &iscritical, &bytes ))
         return NULL;
-
+  
     lc = PyMem_NEW(LDAPControl, 1);
     if (lc == NULL) {
         PyErr_NoMemory();
@@ -107,12 +106,11 @@ Tuple_to_LDAPControl( PyObject* tup )
         berbytes.bv_val = PyBytes_AsString(bytes);
     }
     else {
-	PyErr_SetObject(PyExc_TypeError, Py_BuildValue("sO",
-            "expected a byte-string", bytes));
+        LDAPerror_TypeError("Tuple_to_LDAPControl(): expected bytes", bytes);
         LDAPControl_DEL(lc);
         return NULL;
     }
-
+    
     lc->ldctl_value = berbytes;
 
     return lc;
@@ -128,11 +126,10 @@ LDAPControls_from_object(PyObject* list, LDAPControl ***controls_ret)
     LDAPControl** ldcs;
     LDAPControl* ldc;
     PyObject* item;
-
+  
     if (!PySequence_Check(list)) {
-	PyErr_SetObject(PyExc_TypeError, Py_BuildValue("sO",
-	   "expected a list", list));
-	return 0;
+        LDAPerror_TypeError("LDAPControls_from_object(): expected a list", list);
+        return 0;
     }
 
     len = PySequence_Length(list);
@@ -175,22 +172,21 @@ LDAPControls_to_List(LDAPControl **ldcs)
     if (tmp)
         while (*tmp++) num_ctrls++;
 
-    if (!(res = PyList_New(num_ctrls)))
-        goto endlbl;
+    if ((res = PyList_New(num_ctrls)) == NULL) {
+        return NULL;
+    }
 
     for (i = 0; i < num_ctrls; i++) {
-        if (!(pyctrl = Py_BuildValue("sbO&", ldcs[i]->ldctl_oid,
-                                     ldcs[i]->ldctl_iscritical,
-                                     LDAPberval_to_object,
-                                     &ldcs[i]->ldctl_value))) {
-            goto endlbl;
+        pyctrl = Py_BuildValue("sbO&",
+                               ldcs[i]->ldctl_oid,
+                               ldcs[i]->ldctl_iscritical,
+                               LDAPberval_to_object, &ldcs[i]->ldctl_value);
+        if (pyctrl == NULL) {
+            Py_DECREF(res);
+            return NULL;
         }
         PyList_SET_ITEM(res, i, pyctrl);
     }
-    Py_INCREF(res);
-
- endlbl:
-    Py_XDECREF(res);
     return res;
 }
 
@@ -246,7 +242,7 @@ encode_rfc2696(PyObject *self, PyObject *args)
     BerElement *ber = 0;
     struct berval cookie, *ctrl_val;
     Py_ssize_t cookie_len;
-    unsigned long size;
+    int size = 0; /* ber_int_t is int */
     ber_tag_t tag;
 
     if (!PyArg_ParseTuple(args, "is#:encode_page_control", &size,
@@ -304,7 +300,7 @@ decode_rfc2696(PyObject *self, PyObject *args)
     struct berval ldctl_value;
     ber_tag_t tag;
     struct berval *cookiep;
-    unsigned long count;
+    int count = 0;  /* ber_int_t is int */
     Py_ssize_t ldctl_value_len;
 
     if (!PyArg_ParseTuple(args, "s#:decode_page_control",
@@ -324,7 +320,7 @@ decode_rfc2696(PyObject *self, PyObject *args)
         goto endlbl;
     }
 
-    res = Py_BuildValue("(lO&)", count, LDAPberval_to_object, cookiep);
+    res = Py_BuildValue("(iO&)", count, LDAPberval_to_object, cookiep);
     ber_bvfree(cookiep);
 
  endlbl:
@@ -347,16 +343,33 @@ encode_assertion_control(PyObject *self, PyObject *args)
         goto endlbl;
     }
 
+    /* XXX: ldap_create() is a nasty and slow hack. It's creating a full blown
+     * LDAP object just to encode assertion controls.
+     */
+    Py_BEGIN_ALLOW_THREADS
     err = ldap_create(&ld);
+    Py_END_ALLOW_THREADS
+
     if (err != LDAP_SUCCESS)
     	return LDAPerror(ld, "ldap_create");
 
     err = ldap_create_assertion_control_value(ld,assertion_filterstr,&ctrl_val);
-    if (err != LDAP_SUCCESS)
-    	return LDAPerror(ld, "ldap_create_assertion_control_value");
+
+    if (err != LDAP_SUCCESS) {
+        LDAPerror(ld, "ldap_create_assertion_control_value");
+        Py_BEGIN_ALLOW_THREADS
+        ldap_unbind_ext(ld, NULL, NULL);
+        Py_END_ALLOW_THREADS
+        return NULL;
+    }
+    Py_BEGIN_ALLOW_THREADS
+    ldap_unbind_ext(ld, NULL, NULL);
+    Py_END_ALLOW_THREADS
 
     res = LDAPberval_to_object(&ctrl_val);
-
+    if (ctrl_val.bv_val != NULL) {
+        ber_memfree(ctrl_val.bv_val);
+    }
     endlbl:
 
     return res;
@@ -375,5 +388,3 @@ LDAPinit_control(PyObject *d)
 {
     LDAPadd_methods(d, methods);
 }
-
-
