@@ -1,7 +1,7 @@
 /* See https://www.python-ldap.org/ for details. */
 
 #include "common.h"
-#include "errors.h"
+#include "constants.h"
 #include "LDAPObject.h"
 #include "ldapcontrol.h"
 #include "options.h"
@@ -23,7 +23,7 @@ option_error(int res, const char *fn)
         PyErr_SetString(PyExc_ValueError, "option error");
     else if (res == LDAP_PARAM_ERROR)
         PyErr_SetString(PyExc_ValueError, "parameter error");
-    else if (res == LDAP_NO_MEMORY)
+    else if (res == LDAP_NO_MEMORY) 
         PyErr_NoMemory();
     else
         PyErr_Format(PyExc_SystemError, "error %d from %s", res, fn);
@@ -136,16 +136,44 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
 	    break;
     case LDAP_OPT_TIMEOUT:
     case LDAP_OPT_NETWORK_TIMEOUT:
-	    /* Float valued timeval options */
-	    if (!PyArg_Parse(value, "d:set_option", &doubleval))
-		return 0;
-            if (doubleval >= 0) {
-	        set_timeval_from_double( &tv, doubleval );
-                ptr = &tv;
-            } else {
-    	        ptr = NULL;
+        /* Float valued timeval options */
+        if (value == Py_None) {
+            /* None is mapped to infinity timeout */
+            doubleval = -1;
+        } else {
+            /* 'd' handles int/long */
+            if (!PyArg_Parse(value, "d:set_option", &doubleval)) {
+                if (PyErr_ExceptionMatches(PyExc_TypeError)) {
+                    /* TypeError: mention either float or None is expected */
+                    PyErr_Clear();
+                    PyErr_Format(
+                        PyExc_TypeError,
+                        "A float or None is expected for timeout, got %.100s",
+                        Py_TYPE(value)->tp_name
+                    );
+                }
+                return 0;
             }
-	    break;
+        }
+
+        if (doubleval >= 0) {
+            set_timeval_from_double( &tv, doubleval );
+            ptr = &tv;
+        } else if (doubleval == -1) {
+            /* -1 is infinity timeout */
+            tv.tv_sec = -1;
+            tv.tv_usec = 0;
+            ptr = &tv;
+        } else {
+            PyErr_Format(
+                PyExc_ValueError,
+                "timeout must be >= 0 or -1/None for infinity, got %d",
+                option
+            );
+            return 0;
+        }
+        break;
+
     case LDAP_OPT_SERVER_CONTROLS:
     case LDAP_OPT_CLIENT_CONTROLS:
             if (!LDAPControls_from_object(value, &controls))
@@ -156,14 +184,14 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
 	    PyErr_Format(PyExc_ValueError, "unknown option %d", option);
 	    return 0;
     }
-
+	
     if (self) LDAP_BEGIN_ALLOW_THREADS(self);
     res = ldap_set_option(ld, option, ptr);
     if (self) LDAP_END_ALLOW_THREADS(self);
 
     if ((option == LDAP_OPT_SERVER_CONTROLS) || (option == LDAP_OPT_CLIENT_CONTROLS))
         LDAPControl_List_DEL(controls);
-
+    
     if (res != LDAP_OPT_SUCCESS) {
         option_error(res, "ldap_set_option");
         return 0;
@@ -180,10 +208,9 @@ LDAP_get_option(LDAPObject *self, int option)
     struct timeval *tv;
     LDAPAPIInfo apiinfo;
     LDAPControl **lcs;
-    LDAPControl *lc;
     char *strval;
-    PyObject *extensions, *v, *tup;
-    Py_ssize_t i, num_extensions, num_controls;
+    PyObject *extensions, *v;
+    Py_ssize_t i, num_extensions;
     LDAP *ld;
 
     ld = self ? self->ldap : NULL;
@@ -196,7 +223,7 @@ LDAP_get_option(LDAPObject *self, int option)
 	    if (self) LDAP_END_ALLOW_THREADS(self);
 	    if (res != LDAP_OPT_SUCCESS)
 		return option_error(res, "ldap_get_option");
-
+    
 	    /* put the extensions into tuple form */
 	    num_extensions = 0;
 	    while (apiinfo.ldapai_extensions[num_extensions])
@@ -204,7 +231,7 @@ LDAP_get_option(LDAPObject *self, int option)
 	    extensions = PyTuple_New(num_extensions);
 	    for (i = 0; i < num_extensions; i++)
 		PyTuple_SET_ITEM(extensions, i,
-		    PyBytes_FromString(apiinfo.ldapai_extensions[i]));
+		    PyUnicode_FromString(apiinfo.ldapai_extensions[i]));
 
 	    /* return api info as a dictionary */
 	    v = Py_BuildValue("{s:i, s:i, s:i, s:s, s:i, s:O}",
@@ -321,7 +348,7 @@ LDAP_get_option(LDAPObject *self, int option)
 		Py_INCREF(Py_None);
 		return Py_None;
 	    }
-	    v = PyBytes_FromString(strval);
+	    v = PyUnicode_FromString(strval);
 	    ldap_memfree(strval);
 	    return v;
 
@@ -352,29 +379,10 @@ LDAP_get_option(LDAPObject *self, int option)
 	    if (res != LDAP_OPT_SUCCESS)
 		return option_error(res, "ldap_get_option");
 
-            if (lcs == NULL)
-                return PyList_New(0);
-
-            /* Get the number of controls */
-            num_controls = 0;
-            while (lcs[num_controls])
-                num_controls++;
-
-            /* We'll build a list of controls, with each control a tuple */
-            v = PyList_New(num_controls);
-            for (i = 0; i < num_controls; i++) {
-                lc = lcs[i];
-                tup = Py_BuildValue("(sbs)",
-                                    lc->ldctl_oid,
-                                    lc->ldctl_iscritical,
-                                    lc->ldctl_value.bv_val);
-                PyList_SET_ITEM(v, i, tup);
-            }
-
+            v = LDAPControls_to_List(lcs);
             ldap_controls_free(lcs);
-
             return v;
-
+            
     default:
 	    PyErr_Format(PyExc_ValueError, "unknown option %d", option);
 	    return NULL;
