@@ -5,15 +5,17 @@ Automatic tests for python-ldap's C wrapper module _ldap
 See https://www.python-ldap.org/ for details.
 """
 
+from __future__ import unicode_literals
+
 import os
 import unittest
-from slapdtest import SlapdTestCase
 
 # Switch off processing .ldaprc or ldap.conf before importing _ldap
 os.environ['LDAPNOINIT'] = '1'
 
 # import the plain C wrapper module
 import _ldap
+from slapdtest import SlapdTestCase, requires_tls
 
 
 class TestLdapCExtension(SlapdTestCase):
@@ -49,6 +51,40 @@ class TestLdapCExtension(SlapdTestCase):
             ])
         )
 
+    def setUp(self):
+        super(TestLdapCExtension, self).setUp()
+        self._writesuffix = None
+
+    def tearDown(self):
+        # cleanup test subtree
+        if self._writesuffix is not None:
+            self.server.ldapdelete(self._writesuffix, recursive=True)
+        super(TestLdapCExtension, self).tearDown()
+
+    @property
+    def writesuffix(self):
+        """Initialize writesuffix on demand
+
+        Creates a clean subtree for tests that write to slapd. ldapdelete
+        is not able to delete a Root DSE, therefore we need a temporary
+        work space.
+
+        :return: DN
+        """
+        if self._writesuffix is not None:
+            return self._writesuffix
+        self._writesuffix = 'ou=write tests,%s' % self.server.suffix
+        # Add writeable subtree
+        self.server.ldapadd(
+            "\n".join([
+                'dn: ' + self._writesuffix,
+                'objectClass: organizationalUnit',
+                'ou:' + self._writesuffix.split(',')[0][3:],
+                ''
+            ])
+        )
+        return self._writesuffix
+
     def _open_conn(self, bind=True):
         """
         Starts a server, and returns a LDAPObject bound to it
@@ -64,10 +100,10 @@ class TestLdapCExtension(SlapdTestCase):
         return l
 
     def assertNotNone(self, expr, msg=None):
-        self.failIf(expr is None, msg or repr(expr))
+        self.assertFalse(expr is None, msg or repr(expr))
 
     def assertNone(self, expr, msg=None):
-        self.failIf(expr is not None, msg or repr(expr))
+        self.assertFalse(expr is not None, msg or repr(expr))
 
     # Test for the existence of a whole bunch of constants
     # that the C module is supposed to export
@@ -175,6 +211,21 @@ class TestLdapCExtension(SlapdTestCase):
         self.assertNotNone(_ldap.URL_ERR_BADSCOPE)
         self.assertNotNone(_ldap.URL_ERR_MEM)
 
+    def test_test_flags(self):
+        # test flag, see slapdtest and tox.ini
+        disabled = os.environ.get('CI_DISABLED')
+        if not disabled:
+            self.skipTest("No CI_DISABLED env var")
+        disabled = set(disabled.split(':'))
+        if 'TLS' in disabled:
+            self.assertFalse(_ldap.TLS_AVAIL)
+        else:
+            self.assertFalse(_ldap.TLS_AVAIL)
+        if 'SASL' in disabled:
+            self.assertFalse(_ldap.SASL_AVAIL)
+        else:
+            self.assertFalse(_ldap.SASL_AVAIL)
+
     def test_simple_bind(self):
         l = self._open_conn()
 
@@ -195,7 +246,7 @@ class TestLdapCExtension(SlapdTestCase):
             '',
             _ldap.SCOPE_BASE,
             '(objectClass=*)',
-            ['objectClass', 'namingContexts'],
+            [str('objectClass'), str('namingContexts')],
         )
         self.assertEqual(type(m), type(0))
         result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
@@ -205,9 +256,9 @@ class TestLdapCExtension(SlapdTestCase):
         self.assertEqual(ctrls, [])
         root_dse = pmsg[0][1]
         self.assertTrue('objectClass' in root_dse)
-        self.assertTrue('OpenLDAProotDSE' in root_dse['objectClass'])
+        self.assertTrue(b'OpenLDAProotDSE' in root_dse['objectClass'])
         self.assertTrue('namingContexts' in root_dse)
-        self.assertEqual(root_dse['namingContexts'], [self.server.suffix])
+        self.assertEqual(root_dse['namingContexts'], [self.server.suffix.encode('ascii')])
 
     def test_unbind(self):
         l = self._open_conn()
@@ -235,8 +286,8 @@ class TestLdapCExtension(SlapdTestCase):
         self.assertEqual(len(pmsg[0]), 2)
         self.assertEqual(pmsg[0][0], self.server.suffix)
         self.assertEqual(pmsg[0][0], self.server.suffix)
-        self.assertTrue('dcObject' in pmsg[0][1]['objectClass'])
-        self.assertTrue('organization' in pmsg[0][1]['objectClass'])
+        self.assertTrue(b'dcObject' in pmsg[0][1]['objectClass'])
+        self.assertTrue(b'organization' in pmsg[0][1]['objectClass'])
         self.assertEqual(msgid, m)
         self.assertEqual(ctrls, [])
 
@@ -270,17 +321,24 @@ class TestLdapCExtension(SlapdTestCase):
         self.assertEqual(msgid, m)
         self.assertEqual(ctrls, [])
 
+    def test_invalid_search_filter(self):
+        l = self._open_conn()
+        with self.assertRaises(_ldap.FILTER_ERROR):
+            l.search_ext(
+                self.server.suffix, _ldap.SCOPE_SUBTREE, 'bogus filter expr'
+            )
+
     def test_add(self):
         """
         test add operation
         """
         l = self._open_conn()
         m = l.add_ext(
-            "cn=Foo," + self.server.suffix,
+            "cn=Foo," + self.writesuffix,
             [
-                ('objectClass', 'organizationalRole'),
-                ('cn', 'Foo'),
-                ('description', 'testing'),
+                ('objectClass', b'organizationalRole'),
+                ('cn', b'Foo'),
+                ('description', b'testing'),
             ]
         )
         self.assertEqual(type(m), type(0))
@@ -290,7 +348,7 @@ class TestLdapCExtension(SlapdTestCase):
         self.assertEqual(msgid, m)
         self.assertEqual(ctrls, [])
         # search for it back
-        m = l.search_ext(self.server.suffix, _ldap.SCOPE_SUBTREE, '(cn=Foo)')
+        m = l.search_ext(self.writesuffix, _ldap.SCOPE_SUBTREE, '(cn=Foo)')
         result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         # Expect to get the objects
         self.assertEqual(result, _ldap.RES_SEARCH_RESULT)
@@ -300,11 +358,11 @@ class TestLdapCExtension(SlapdTestCase):
         self.assertEqual(
             pmsg[0],
             (
-                'cn=Foo,'+self.server.suffix,
+                'cn=Foo,'+self.writesuffix,
                 {
-                    'objectClass': ['organizationalRole'],
-                    'cn': ['Foo'],
-                    'description': ['testing'],
+                    'objectClass': [b'organizationalRole'],
+                    'cn': [b'Foo'],
+                    'description': [b'testing'],
                 }
             )
         )
@@ -315,14 +373,14 @@ class TestLdapCExtension(SlapdTestCase):
         """
         l = self._open_conn()
         # first, add an object with a field we can compare on
-        dn = "cn=CompareTest," + self.server.suffix
+        dn = "cn=CompareTest," + self.writesuffix
         m = l.add_ext(
             dn,
             [
-                ('objectClass', 'person'),
-                ('sn', 'CompareTest'),
-                ('cn', 'CompareTest'),
-                ('userPassword', 'the_password'),
+                ('objectClass', b'person'),
+                ('sn', b'CompareTest'),
+                ('cn', b'CompareTest'),
+                ('userPassword', b'the_password'),
             ],
         )
         self.assertEqual(type(m), type(0))
@@ -369,12 +427,12 @@ class TestLdapCExtension(SlapdTestCase):
     def test_delete(self):
         l = self._open_conn()
         # first, add an object we will delete
-        dn = "cn=Deleteme,"+self.server.suffix
+        dn = "cn=Deleteme,"+self.writesuffix
         m = l.add_ext(
             dn,
             [
-                ('objectClass', 'organizationalRole'),
-                ('cn', 'Deleteme'),
+                ('objectClass', b'organizationalRole'),
+                ('cn', b'Deleteme'),
             ]
         )
         self.assertEqual(type(m), type(0))
@@ -393,9 +451,9 @@ class TestLdapCExtension(SlapdTestCase):
 
         # try deleting an object that doesn't exist
         m = l.modify_ext(
-            "cn=DoesNotExist,"+self.server.suffix,
+            "cn=DoesNotExist,"+self.writesuffix,
             [
-                (_ldap.MOD_ADD, 'description', ['blah']),
+                (_ldap.MOD_ADD, 'description', [b'blah']),
             ]
         )
         try:
@@ -413,7 +471,7 @@ class TestLdapCExtension(SlapdTestCase):
         m = l.modify_ext(
             "cn=DoesNotExist,"+self.server.suffix,
             [
-                (_ldap.MOD_ADD, 'description', ['dummy']),
+                (_ldap.MOD_ADD, 'description', [b'dummy']),
             ]
         )
         self.assertTrue(isinstance(m, int))
@@ -430,14 +488,14 @@ class TestLdapCExtension(SlapdTestCase):
         """
         l = self._open_conn()
         # first, add an object we will delete
-        dn = "cn=AddToMe,"+self.server.suffix
+        dn = "cn=AddToMe,"+self.writesuffix
         m = l.add_ext(
             dn,
             [
-                ('objectClass', 'person'),
-                ('cn', 'AddToMe'),
-                ('sn', 'Modify'),
-                ('description', 'a description'),
+                ('objectClass', b'person'),
+                ('cn', b'AddToMe'),
+                ('sn', b'Modify'),
+                ('description', b'a description'),
             ]
         )
         self.assertEqual(type(m), type(0))
@@ -447,7 +505,7 @@ class TestLdapCExtension(SlapdTestCase):
         m = l.modify_ext(
             dn,
             [
-                (_ldap.MOD_ADD, 'description', ['b desc', 'c desc']),
+                (_ldap.MOD_ADD, 'description', [b'b desc', b'c desc']),
             ]
         )
         result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
@@ -456,7 +514,7 @@ class TestLdapCExtension(SlapdTestCase):
         self.assertEqual(msgid, m)
         self.assertEqual(ctrls, [])
         # search for it back
-        m = l.search_ext(self.server.suffix, _ldap.SCOPE_SUBTREE, '(cn=AddToMe)')
+        m = l.search_ext(self.writesuffix, _ldap.SCOPE_SUBTREE, '(cn=AddToMe)')
         result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         # Expect to get the objects
         self.assertEqual(result, _ldap.RES_SEARCH_RESULT)
@@ -466,16 +524,16 @@ class TestLdapCExtension(SlapdTestCase):
         self.assertEqual(pmsg[0][0], dn)
         d = list(pmsg[0][1]['description'])
         d.sort()
-        self.assertEqual(d, ['a description', 'b desc', 'c desc'])
+        self.assertEqual(d, [b'a description', b'b desc', b'c desc'])
 
     def test_rename(self):
         l = self._open_conn()
-        dn = "cn=RenameMe,"+self.server.suffix
+        dn = "cn=RenameMe,"+self.writesuffix
         m = l.add_ext(
             dn,
             [
-                ('objectClass', 'organizationalRole'),
-                ('cn', 'RenameMe'),
+                ('objectClass', b'organizationalRole'),
+                ('cn', b'RenameMe'),
             ]
         )
         self.assertEqual(type(m), type(0))
@@ -491,7 +549,7 @@ class TestLdapCExtension(SlapdTestCase):
         self.assertEqual(ctrls, [])
 
         # make sure the old one is gone
-        m = l.search_ext(self.server.suffix, _ldap.SCOPE_SUBTREE, '(cn=RenameMe)')
+        m = l.search_ext(self.writesuffix, _ldap.SCOPE_SUBTREE, '(cn=RenameMe)')
         result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEqual(result, _ldap.RES_SEARCH_RESULT)
         self.assertEqual(len(pmsg), 0) # expect no results
@@ -499,23 +557,23 @@ class TestLdapCExtension(SlapdTestCase):
         self.assertEqual(ctrls, [])
 
         # check that the new one looks right
-        dn2 = "cn=IAmRenamed,"+self.server.suffix
-        m = l.search_ext(self.server.suffix, _ldap.SCOPE_SUBTREE, '(cn=IAmRenamed)')
+        dn2 = "cn=IAmRenamed,"+self.writesuffix
+        m = l.search_ext(self.writesuffix, _ldap.SCOPE_SUBTREE, '(cn=IAmRenamed)')
         result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEqual(result, _ldap.RES_SEARCH_RESULT)
         self.assertEqual(msgid, m)
         self.assertEqual(ctrls, [])
         self.assertEqual(len(pmsg), 1)
         self.assertEqual(pmsg[0][0], dn2)
-        self.assertEqual(pmsg[0][1]['cn'], ['IAmRenamed'])
+        self.assertEqual(pmsg[0][1]['cn'], [b'IAmRenamed'])
 
         # create the container
-        containerDn = "ou=RenameContainer,"+self.server.suffix
+        containerDn = "ou=RenameContainer,"+self.writesuffix
         m = l.add_ext(
             containerDn,
             [
-                ('objectClass', 'organizationalUnit'),
-                ('ou', 'RenameContainer'),
+                ('objectClass', b'organizationalUnit'),
+                ('ou', b'RenameContainer'),
             ]
         )
         result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
@@ -533,25 +591,25 @@ class TestLdapCExtension(SlapdTestCase):
         self.assertEqual(ctrls, [])
 
         # make sure dn2 is gone
-        m = l.search_ext(self.server.suffix, _ldap.SCOPE_SUBTREE, '(cn=IAmRenamed)')
+        m = l.search_ext(self.writesuffix, _ldap.SCOPE_SUBTREE, '(cn=IAmRenamed)')
         result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEqual(result, _ldap.RES_SEARCH_RESULT)
         self.assertEqual(len(pmsg), 0) # expect no results
         self.assertEqual(msgid, m)
         self.assertEqual(ctrls, [])
 
-        m = l.search_ext(self.server.suffix, _ldap.SCOPE_SUBTREE, '(objectClass=*)')
+        m = l.search_ext(self.writesuffix, _ldap.SCOPE_SUBTREE, '(objectClass=*)')
         result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
 
         # make sure dn3 is there
-        m = l.search_ext(self.server.suffix, _ldap.SCOPE_SUBTREE, '(cn=IAmRenamedAgain)')
+        m = l.search_ext(self.writesuffix, _ldap.SCOPE_SUBTREE, '(cn=IAmRenamedAgain)')
         result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEqual(result, _ldap.RES_SEARCH_RESULT)
         self.assertEqual(msgid, m)
         self.assertEqual(ctrls, [])
         self.assertEqual(len(pmsg), 1)
         self.assertEqual(pmsg[0][0], dn3)
-        self.assertEqual(pmsg[0][1]['cn'], ['IAmRenamedAgain'])
+        self.assertEqual(pmsg[0][1]['cn'], [b'IAmRenamedAgain'])
 
 
     def test_whoami(self):
@@ -576,17 +634,24 @@ class TestLdapCExtension(SlapdTestCase):
         r = l.whoami_s()
         self.assertEqual("", r)
 
+    def test_whoami_after_unbind(self):
+        # https://github.com/pyldap/pyldap/issues/29
+        l = self._open_conn(bind=True)
+        l.unbind_ext()
+        with self.assertRaises(_ldap.LDAPError):
+            l.whoami_s()
+
     def test_passwd(self):
         l = self._open_conn()
         # first, create a user to change password on
-        dn = "cn=PasswordTest," + self.server.suffix
+        dn = "cn=PasswordTest," + self.writesuffix
         m = l.add_ext(
             dn,
             [
-                ('objectClass', 'person'),
-                ('sn', 'PasswordTest'),
-                ('cn', 'PasswordTest'),
-                ('userPassword', 'initial'),
+                ('objectClass', b'person'),
+                ('sn', b'PasswordTest'),
+                ('cn', b'PasswordTest'),
+                ('userPassword', b'initial'),
             ]
         )
         self.assertEqual(type(m), type(0))
@@ -667,12 +732,6 @@ class TestLdapCExtension(SlapdTestCase):
             return
         # TODO
 
-    def test_tls(self):
-        l = self._open_conn()
-        if not self._require_attr(l, 'start_tls_s'):    # HAVE_TLS
-            return
-        # TODO
-
     def test_cancel(self):
         l = self._open_conn()
         if not self._require_attr(l, 'cancel'):         # FEATURE_CANCEL
@@ -717,6 +776,114 @@ class TestLdapCExtension(SlapdTestCase):
         else:
             self.fail("expected INVALID_CREDENTIALS, got %r" % r)
 
+    # TODO: test_extop
+
+    def assertInvalidControls(self, func, *args, **kwargs):
+        post = kwargs.pop('post', ())
+        self.assertFalse(kwargs)
+        # last two args are serverctrls, clientctrls
+        with self.assertRaises(TypeError) as e:
+            func(*(args + (object, None) + post))
+        self.assertEqual(
+            e.exception.args,
+            ('LDAPControls_from_object(): expected a list', object)
+        )
+        with self.assertRaises(TypeError) as e:
+            func(*(args + (None, object) + post))
+        self.assertEqual(
+            e.exception.args,
+            ('LDAPControls_from_object(): expected a list', object)
+        )
+
+    def test_invalid_controls(self):
+        l = self._open_conn()
+        self.assertInvalidControls(l.simple_bind, "", "")
+        self.assertInvalidControls(l.whoami_s)
+        self.assertInvalidControls(l.passwd, 'dn', 'initial', 'changed')
+        self.assertInvalidControls(l.add_ext, 'dn', [('cn', b'cn')])
+        self.assertInvalidControls(
+            l.modify_ext, 'dn', [(_ldap.MOD_ADD, 'attr', [b'value'])])
+        self.assertInvalidControls(l.compare_ext, 'dn', 'val1', 'val2')
+        self.assertInvalidControls(
+            l.rename, 'dn', 'newdn', 'container', False)
+        self.assertInvalidControls(
+            l.search_ext, 'dn', _ldap.SCOPE_SUBTREE, '(objectClass=*)',
+            None, 1)
+        self.assertInvalidControls(l.delete_ext, 'dn')
+        m = l.search_ext(
+            self.server.suffix, _ldap.SCOPE_SUBTREE, '(objectClass=*)')
+        self.assertInvalidControls(l.abandon_ext, m)
+        self.assertInvalidControls(l.cancel, 0)
+        self.assertInvalidControls(l.extop, 'oid', 'value')
+        if hasattr(l, 'sasl_bind_s'):
+            self.assertInvalidControls(l.sasl_bind_s, 'dn', 'MECH', 'CRED')
+        if hasattr(l, 'sasl_interactive_bind_s'):
+            self.assertInvalidControls(
+                l.sasl_interactive_bind_s, 'who', 'SASLObject', post=(1,))
+        self.assertInvalidControls(l.unbind_ext)
+
+    @requires_tls()
+    def test_tls_ext(self):
+        l = self._open_conn(bind=False)
+        # StartTLS needs LDAPv3
+        l.set_option(_ldap.OPT_PROTOCOL_VERSION, _ldap.VERSION3)
+        l.set_option(_ldap.OPT_X_TLS_CACERTFILE, self.server.cafile)
+        # re-create TLS context
+        l.set_option(_ldap.OPT_X_TLS_NEWCTX, 0)
+        l.start_tls_s()
+
+    @requires_tls()
+    def test_tls_require_cert(self):
+        # libldap defaults to secure cert validation
+        # see libraries/libldap/init.c
+        #     gopts->ldo_tls_require_cert = LDAP_OPT_X_TLS_DEMAND;
+
+        self.assertEqual(
+            _ldap.get_option(_ldap.OPT_X_TLS_REQUIRE_CERT),
+            _ldap.OPT_X_TLS_DEMAND
+        )
+        l = self._open_conn(bind=False)
+        self.assertEqual(
+            l.get_option(_ldap.OPT_X_TLS_REQUIRE_CERT),
+            _ldap.OPT_X_TLS_DEMAND
+        )
+
+    @requires_tls()
+    def test_tls_ext_noca(self):
+        l = self._open_conn(bind=False)
+        l.set_option(_ldap.OPT_PROTOCOL_VERSION, _ldap.VERSION3)
+        # fails because libldap defaults to secure cert validation but
+        # the test CA is not installed as trust anchor.
+        with self.assertRaises(_ldap.CONNECT_ERROR) as e:
+            l.start_tls_s()
+        # known resaons:
+        # Ubuntu on Travis: '(unknown error code)'
+        # OpenSSL 1.1: error:1416F086:SSL routines:\
+        #    tls_process_server_certificate:certificate verify failed
+        # NSS: TLS error -8172:Peer's certificate issuer has \
+        #    been marked as not trusted by the user.
+        msg = str(e.exception)
+        candidates = ('certificate', 'tls', '(unknown error code)')
+        if not any(s in msg.lower() for s in candidates):
+            self.fail(msg)
+
+    @requires_tls()
+    def test_tls_ext_clientcert(self):
+        l = self._open_conn(bind=False)
+        l.set_option(_ldap.OPT_PROTOCOL_VERSION, _ldap.VERSION3)
+        l.set_option(_ldap.OPT_X_TLS_CACERTFILE, self.server.cafile)
+        l.set_option(_ldap.OPT_X_TLS_CERTFILE, self.server.clientcert)
+        l.set_option(_ldap.OPT_X_TLS_KEYFILE, self.server.clientkey)
+        l.set_option(_ldap.OPT_X_TLS_REQUIRE_CERT, _ldap.OPT_X_TLS_HARD)
+        l.set_option(_ldap.OPT_X_TLS_NEWCTX, 0)
+        l.start_tls_s()
+
+    @requires_tls()
+    def test_tls_packages(self):
+        # libldap has tls_g.c, tls_m.c, and tls_o.c with ldap_int_tls_impl
+        package = _ldap.get_option(_ldap.OPT_X_TLS_PACKAGE)
+        self.assertIn(package, {"GnuTLS", "MozNSS", "OpenSSL"})
+
 
 if __name__ == '__main__':
     unittest.main()
