@@ -5,11 +5,13 @@ Automatic tests for python-ldap's module ldap.cidict
 See https://www.python-ldap.org/ for details.
 """
 
-# from Python's standard lib
+import os
 import unittest
 
-# from python-ldap
-import ldap, ldap.cidict
+# Switch off processing .ldaprc or ldap.conf before importing _ldap
+os.environ['LDAPNOINIT'] = '1'
+import ldap
+import ldap.cidict
 
 
 class TestCidict(unittest.TestCase):
@@ -32,16 +34,17 @@ class TestCidict(unittest.TestCase):
         cix["xYZ"] = 987
         self.assertEqual(cix["XyZ"], 987)
         self.assertEqual(cix.get("xyz", None), 987)
-        cix_keys = cix.keys()
-        cix_keys.sort()
+        cix_keys = sorted(cix.keys())
         self.assertEqual(cix_keys, ['AbCDeF','xYZ'])
-        cix_items = cix.items()
-        cix_items.sort()
+        cix_items = sorted(cix.items())
         self.assertEqual(cix_items, [('AbCDeF',123), ('xYZ',987)])
         del cix["abcdEF"]
-        self.assertEqual("abcdef" in cix, False)
+        self.assertEqual("abcdef" in cix._keys, False)
         self.assertEqual("AbCDef" in cix._keys, False)
+        self.assertEqual("abcdef" in cix, False)
+        self.assertEqual("AbCDef" in cix, False)
         self.assertEqual(cix.has_key("abcdef"), False)
+        self.assertEqual(cix.has_key("AbCDef"), False)
 
 
 if __name__ == '__main__':
