@@ -5,16 +5,20 @@ Automatic tests for python-ldap's module ldif
 See https://www.python-ldap.org/ for details.
 """
 
-# from Python's standard lib
-import unittest
+from __future__ import unicode_literals
+
+import os
 import textwrap
+import unittest
 
 try:
     from StringIO import StringIO
 except ImportError:
     from io import StringIO
 
-# from python-ldap
+# Switch off processing .ldaprc or ldap.conf before importing _ldap
+os.environ['LDAPNOINIT'] = '1'
+
 import ldif
 
 
@@ -184,7 +188,7 @@ class TestEntryRecords(TestLDIFParser):
              value
             attrib2: %s
 
-            """ % (b'asdf.'*20), [
+            """ % ('asdf.'*20), [
                 (
                     'cn=x,cn=y,cn=z',
                     {
@@ -273,6 +277,26 @@ class TestEntryRecords(TestLDIFParser):
         )
 
     def test_unicode(self):
+        # Encode "Ströder" as UTF-8+Base64
+        # Putting "Ströder" in a single line would be an invalid LDIF file
+        # per https://tools.ietf.org/html/rfc2849 (only safe ascii is allowed in a file)
+        self.check_records(
+            """
+            dn: cn=Michael Stroeder,dc=stroeder,dc=com
+            lastname:: U3Ryw7ZkZXI=
+
+            """,
+            [
+                (
+                    'cn=Michael Stroeder,dc=stroeder,dc=com',
+                    {'lastname': [b'Str\303\266der']},
+                ),
+            ]
+        )
+
+    def test_unencoded_unicode(self):
+        # Encode "Ströder" as UTF-8, without base64
+        # This is an invalid LDIF file, but such files are often found in the wild.
         self.check_records(
             """
             dn: cn=Michael Stroeder,dc=stroeder,dc=com
